/*
 *  Boa, an http server
 *  Copyright (C) 1995 Paul Phillips <paulp@go2net.com>
 *  Some changes Copyright (C) 1996 Charles F. Randall <crandall@goldsys.com>
 *  Some changes Copyright (C) 1996 Larry Doolittle <ldoolitt@boa.org>
 *  Some changes Copyright (C) 1996-2002 Jon Nelson <jnelson@boa.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* $Id: boa.c,v 1.2 2008/05/19 11:39:41 kathy Exp $*/
#include <sys/ioctl.h>
#include <net/if.h>


#include "boa.h"
#include "nvram.h"
#include "rtmp_type.h"
#include "oid.h"


/* globals */
int backlog = SO_MAXCONN;
time_t start_time;

int  debug_level = 0;
int  sighup_flag = 0;            /* 1 => signal has happened, needs attention */
int  sigchld_flag = 0;           /* 1 => signal has happened, needs attention */
int  sigalrm_flag = 0;           /* 1 => signal has happened, needs attention */
int  sigterm_flag = 0;           /* lame duck mode */
time_t current_time;
int  pending_requests = 0;
int  reboot_flag = 0;
int  redirectSetupPage = 0;

#ifdef RALINK_ATE
int  ATE_ACTION_MODE;
int  ATE_ACTION_START;
char ateda[18];
char atesa[18];
char atebssid[18];
char atetxpow;
int  atechannel;
char atetxfreqoffset;
int  atetxlen;
long atetxcnt;
int  atetxrate;
int  eeprom_read_write;
int  eeprom_offset;
char eeprom_value[5];
int  mac_read_write;
int  mac_offset;
char mac_value[9];
int  bbp_read_write;
int  bbp_offset;
char bbp_value[3];
#endif

extern const char *config_file_name;
extern INT dev_idx;
extern INT num_of_dev;
extern RT_DEVICE_CARD_INFO dev_info[MAX_DEVICE_NUMBER];




extern int OidQueryInformation(unsigned long OidQueryCode, int socket_id, char *DeviceName, void *ptr, unsigned long PtrLength);
extern int OidSetInformation(unsigned long OidQueryCode, int socket_id, char *DeviceName, void *ptr, unsigned long PtrLength);
extern int reloadWirelessModule();

/* static to boa.c */
static void usage(const char *programname);
static void parse_commandline(int argc, char *argv[]);
static void fixup_server_root(void);
static int create_server_socket(void);
static void drop_privs(void);

/* kathy */
void iw_enum_devices_for_boa(int socket);
void getInitValueOfAdvancePage();
void getInitValueOfWMMPage();
void getInitValueOfMeshConfigPage();
void findRT28xxCardDat(int multicard_support, int skt);


static int sock_opt = 1;
static int do_fork = 1;

int main(int argc, char *argv[])
{
    int server_s;               /* boa socket */
    pid_t pid;
	int s, i;
	char tmp[16];
#ifndef RT61AP  //add by kathy
	char *install_dir=NULL;
	char tmp_make_config_file[120];
	char tmp_nvram[80];
	char tmp_mac_nvram[80];
	char tmp_sys_buf[80];
	FILE *fh;
	char *wordlist = NULL;
	UCHAR multicard_support=0;

#endif

    /* add by arvin tai 2004.12.21 */
    nvram_init();
#ifdef RT61AP  //add by kathy
    start_wan_service();
    start_lan_service();
    web_check_config_file();
#endif
    /* add end */

	//kathy
	headerProfileSetting = currentProfileSetting = selectedProfileSetting = NULL;

    /* set umask to u+rw, u-x, go-rwx */
    if (umask(077) < 0) {
        perror("umask");
        exit(EXIT_FAILURE);
    }

    /* but first, update timestamp, because log_error_time uses it */
    (void) time(&current_time);

    /* set timezone right away */
    tzset();
#if 1
    {
        int devnullfd = -1;
        devnullfd = open("/dev/null", 0);

        /* make STDIN point to /dev/null */
        if (devnullfd == -1) {
            DIE("can't open /dev/null");
        }

        if (dup2(devnullfd, STDIN_FILENO) == -1) {
            DIE("can't dup2 /dev/null to STDIN_FILENO");
        }

        (void) close(devnullfd);
    }
#endif // if 0
    parse_commandline(argc,argv);
    fixup_server_root();
    create_common_env();
    read_config_files();
    open_logs();
    server_s = create_server_socket();
    init_signals();
    build_needs_escape();

    /* background ourself */
    if (do_fork) {
        pid = fork();
    } else {
        pid = getpid();
    }

    switch (pid) {
    case -1:
        /* error */
        perror("fork/getpid");
        exit(EXIT_FAILURE);
    case 0:
        /* child, success */
        break;
    default:
        /* parent, success */
        if (pid_file != NULL) {
            FILE *PID_FILE = fopen(pid_file, "w");
            if (PID_FILE != NULL) {
                fprintf(PID_FILE, "%d", pid);
                fclose(PID_FILE);
            } else {
                perror("fopen pid file");
            }
        }

        if (do_fork)
            exit(EXIT_SUCCESS);
        break;
    }

    drop_privs();
    /* main loop */
    timestamp();

    status.requests = 0;
    status.errors = 0;

    start_time = current_time;
#if 1
	s = socket(AF_INET, SOCK_DGRAM, 0);

	memset(tmp,0x00, sizeof(tmp));
	memset(dev_info,0x00, sizeof(dev_info));

	iw_enum_devices_for_boa(s);

	for (i=0 ; i<8; i++)
	{
		if (strcmp((char *)dev_info[i].Name, "") == 0)
			break;
		
		if (OidQueryInformation(RT_OID_DRIVER_DEVICE_NAME , s, (char *)dev_info[i].Name, &tmp[0], 16) < 0)
		{
			fprintf(stderr, "Query RT_OID_DRIVER_DEVICE_NAME error! i=%d, tmp=%s, dev_name=%s\n", i, &tmp[0], dev_info[i].Name);
			continue;
		}
	}

	//no fine ra number from driver
	if (i == 8)
		strcpy((char *)dev_info[0].Name, DEF_IFNAME);


	// Multiple Card Support
	if (OidQueryInformation(RT_OID_QUERY_MULTIPLE_CARD_SUPPORT , s, (char *)dev_info[dev_idx].Name, &multicard_support, sizeof(multicard_support)) < 0)
	{
		fprintf(stderr, "Query RT_OID_QUERY_MULTIPLE_CARD_SUPPORT error! i=%d, multicard_support=%s, dev_name=%s\n", i, &multicard_support, dev_info[i].Name);
		multicard_support = 0;
	}
	
	memset(tmp_make_config_file, 0x00, sizeof(tmp_make_config_file));
	memset(tmp_nvram, 0x00, sizeof(tmp_nvram));
	memset(tmp_mac_nvram, 0x00, sizeof(tmp_mac_nvram));

	/* to fine RT28xxCard.dat, if not, create it.*/
	findRT28xxCardDat(multicard_support, s);
	
	/*	1.to find the nvram file name if INSTALL_DIR be set
		2.if support multiple card that must to create mac_nvram.dat
	*/

	/*
	staMultiCardIdx not only be uses in Multiple_Card mode
	if supported MESH mode, interface number have 2 at least: ra0 and mesh0

	if the CardIdx isn't exist, dev_idx = 0.
	*/
	install_dir = getenv("INSTALL_DIR");

	wordlist = nvram_get("staMultiCardName");
	if (strcmp(wordlist, "") == 0)
		dev_idx = 0;
	else
	{
		i = 0;
		while (i <= dev_idx)
		{
			if(strcmp((char *)dev_info[i].Name, wordlist) == 0)
			{
				dev_idx = i;
				break;
			}
			i++;
		}
	}

	
#ifdef USE_DEBUG
	fprintf(stderr, "main :dev_idx=%d,  dev_info[dev_idx].Name=%s\n", dev_idx, dev_info[dev_idx].Name);
#endif 
	nvram_set("staMultiCardName", (char *)dev_info[dev_idx].Name);

#ifdef WAPI_SUPPORT
	nvram_set("staConnectedWAPIIFName", (char *)dev_info[dev_idx].Name);
#endif

#if 0
	wordlist = nvram_get("staMultiCardIdx");
	if (wordlist == NULL)
		tmp_idx=0;
	else
		tmp_idx = atoi(wordlist);
	
	if ( tmp_idx <= dev_idx)
		dev_idx = tmp_idx;
	else
	{
		dev_idx = 0;
		nvram_set("staMultiCardIdx", "0");
	}
#endif	

	if (install_dir == NULL)
	{
		char *pwd=NULL;
		pwd = getenv("PWD");

		sprintf(tmp_nvram, "/etc/nvram.dat");
		if (multicard_support)
		{
			sprintf(tmp_mac_nvram, "/etc/%s_nvram.dat", dev_info[dev_idx].Mac);
		}
		sprintf(tmp_make_config_file, "%s/../../nvram/make_config_file", pwd);
	}
	else
	{
		sprintf(tmp_nvram, "%s/etc/nvram.dat", install_dir);
		if (multicard_support)
		{
			sprintf(tmp_mac_nvram, "%s/etc/%s_nvram.dat", install_dir, dev_info[dev_idx].Mac);
		}
		sprintf(tmp_make_config_file, "%s/WebUI/nvram/make_config_file", tmp_make_config_file);
	}

	if(strlen(tmp_make_config_file) > 120)
	{
		fprintf(stderr, "Path length is more than 120.\n");
		return 0;
	}


	/* 
	create nvram.dat
	*/
	fh = fopen(tmp_nvram, "r");
	if (fh == NULL)
		system(tmp_make_config_file);
	else
		fclose(fh);

	if (!multicard_support)
	{
		// get the init value
		getInitValueOfAdvancePage();
		system(tmp_make_config_file);
	}
	else
	{
		fh = fopen(tmp_mac_nvram, "r");
		if(fh != NULL)
		{
			sprintf(tmp_sys_buf, "cp %s %s", tmp_mac_nvram, tmp_nvram);
			system(tmp_sys_buf);
			fclose(fh);

			// get the init value and update the driver valur into nvram.dat
			getInitValueOfAdvancePage();
		}
		else
		{
			// get the init value
			getInitValueOfAdvancePage();
			sprintf(tmp_sys_buf, "cp %s %s", tmp_nvram, tmp_mac_nvram);
			system(tmp_sys_buf);
		}
	
		i=0;
		while(i < 8)
		{
			memset(tmp_mac_nvram, 0x00, sizeof(tmp_mac_nvram));

			if (strlen((char *)dev_info[i].Mac) != 17)
				break;

			if (install_dir == NULL)
			{
				sprintf(tmp_mac_nvram, "/etc/%s_nvram.dat", dev_info[i].Mac);
			}
			else
			{
				sprintf(tmp_mac_nvram, "%s/etc/%s_nvram.dat", install_dir, dev_info[i].Mac);
			}

			fh = fopen(tmp_mac_nvram, "r");
			if(fh == NULL)
			{
				sprintf(tmp_sys_buf, "cp %s %s", tmp_nvram, tmp_mac_nvram);
				system(tmp_sys_buf);
			}
			else
				fclose(fh);

			i++;
		}
	}
	close(s);

#endif

    loop(server_s);
    return 0;
}

static void usage(const char *programname)
{
    fprintf(stderr, "Usage: %s [-c serverroot] [-d] [-f configfile] [-r chroot] [-u documentroot] [-p portnumber]%s\n",
	    programname,
#ifndef DISABLE_DEBUG
	    " [-l debug_level]"
#else
	    ""
#endif
	   );
#ifndef DISABLE_DEBUG
    print_debug_usage();
#endif
    exit(EXIT_FAILURE);

}

static void parse_commandline(int argc, char *argv[])
{
    int c;                      /* command line arg */

    while ((c = getopt(argc, argv, "c:dl:f:r:p:u:")) != -1) { // add p for port number
	switch (c) {
		case 'c':
			if (server_root)
				free(server_root);
			server_root = strdup(optarg);
			if (!server_root) {
				perror("strdup (for server_root)");
				exit(EXIT_FAILURE);
			}
			break;
		case 'd':
			do_fork = 0;
			break;
		case 'f':
			config_file_name = optarg;
			break;
		case 'r':
			if (chdir(optarg) == -1) {
				log_error_time();
				perror("chdir (to chroot)");
				exit(EXIT_FAILURE);
			}
			if (chroot(optarg) == -1) {
				log_error_time();
				perror("chroot");
				exit(EXIT_FAILURE);
			}
			if (chdir("/") == -1) {
				log_error_time();
				perror("chdir (after chroot)");
				exit(EXIT_FAILURE);
			}
			break;
	#ifndef DISABLE_DEBUG
		case 'l':
			parse_debug(optarg);
			break;
	#endif
		// add config port number from user mode
		case 'p':
			user_server_port = atoi(optarg);
			break;
		case 'u':
			user_document_root = strdup(optarg);
			break;
		default:
			usage(argv[0]);
			exit(EXIT_FAILURE);
		}
    }
}

static int create_server_socket(void)
{
    int server_s;

    server_s = socket(SERVER_PF, SOCK_STREAM, IPPROTO_TCP);
    if (server_s == -1) {
        DIE("unable to create socket");
    }

    /* server socket is nonblocking */
    if (set_nonblock_fd(server_s) == -1) {
        DIE("fcntl: unable to set server socket to nonblocking");
    }

    /* close server socket on exec so cgi's can't write to it */
    if (fcntl(server_s, F_SETFD, 1) == -1) {
        DIE("can't set close-on-exec on server socket!");
    }

    /* reuse socket addr */
    if ((setsockopt(server_s, SOL_SOCKET, SO_REUSEADDR, (void *) &sock_opt,
                    sizeof (sock_opt))) == -1) {
        DIE("setsockopt");
    }

    /* Internet family-specific code encapsulated in bind_server()  */
	fprintf(stderr, "server_s=%d, server_ip=%s, server_port=%d\n", server_s, server_ip, server_port);
    if (bind_server(server_s, server_ip, server_port) == -1) {
        DIE("unable to bind");
    }

    /* listen: large number just in case your kernel is nicely tweaked */
    if (listen(server_s, backlog) == -1) {
        DIE("unable to listen");
    }
    return server_s;
}

static void drop_privs(void)
{
    /* give away our privs if we can */
    if (getuid() == 0) {
        struct passwd *passwdbuf;
        passwdbuf = getpwuid(server_uid);
        if (passwdbuf == NULL) {
            DIE("getpwuid");
        }
        if (initgroups(passwdbuf->pw_name, passwdbuf->pw_gid) == -1) {
            DIE("initgroups");
        }
        if (setgid(server_gid) == -1) {
            DIE("setgid");
        }
        if (setuid(server_uid) == -1) {
            DIE("setuid");
        }
        /* test for failed-but-return-was-successful setuid
         * http://www.securityportal.com/list-archive/bugtraq/2000/Jun/0101.html
         */
        if (server_uid != 0 && setuid(0) != -1) {
            DIE("icky Linux kernel bug!");
        }
    } else {
        if (server_gid || server_uid) {
            log_error_time();
            fprintf(stderr, "Warning: "
                    "Not running as root: no attempt to change"
                    " to uid %u gid %u\n", server_uid, server_gid);
        }
        server_gid = getgid();
        server_uid = getuid();
    }
}

/*
 * Name: fixup_server_root
 *
 * Description: Makes sure the server root is valid.
 *
 */

static void fixup_server_root()
{
    if (!server_root) {
#ifdef SERVER_ROOT
        server_root = strdup(SERVER_ROOT);
        if (!server_root) {
            perror("strdup (SERVER_ROOT)");
            exit(EXIT_FAILURE);
        }
#else
        fputs("boa: don't know where server root is.  Please #define "
              "SERVER_ROOT in boa.h\n"
              "and recompile, or use the -c command line option to "
              "specify it.\n", stderr);
        exit(EXIT_FAILURE);
#endif
    }

    if (chdir(server_root) == -1) {
        fprintf(stderr, "Could not chdir to \"%s\": aborting\n",
                server_root);
        exit(EXIT_FAILURE);
    }
}

/* reference from iwlib.c*/

/*------------------------------------------------------------------*/
/*
 * Extract the interface name out of /proc/net/wireless or /proc/net/dev.
 */
static inline char *
iw_get_ifname_for_boa(char *	name,	/* Where to store the name */
	      int	nsize,	/* Size of name buffer */
	      char *	buf)	/* Current position in buffer */
{
	char *end;

	/* Skip leading spaces */
	while(isspace(*buf))
		buf++;

	/* Get name up to ": "
	* Note : we compare to ": " to make sure to process aliased interfaces
	* properly. Doesn't work on /proc/net/dev, because it doesn't guarantee
	* a ' ' after the ':'*/
	//end = strstr(buf, ": ");
	end = strstr(buf, ":");

	/* Not found ??? To big ??? */
	if((end == NULL) || (((end - buf) + 1) > nsize))
		return(NULL);

	/* Copy */
	memcpy(name, buf, (end - buf));
	name[end - buf] = '\0';

	/* Return value currently unused, just make sure it's non-NULL */
	return(end);
}


void
iw_enum_devices_for_boa(int socket)
{
	char		buff[1024];
	FILE *	fh;
	struct ifconf ifc;
	struct ifreq *ifr;
	int		i;

	/* Check if /proc/net/wireless is available */

	fh = fopen(PROC_NET_WIRELESS, "r");


	if(fh != NULL)
	{
		NDIS_802_11_MAC_ADDRESS CurrentAddress;
		//struct ifreq ifr;
		int ret;

		/* Success : use data from /proc/net/wireless */

		/* Eat 2 lines of header */
		fgets(buff, sizeof(buff), fh);
		fgets(buff, sizeof(buff), fh);

		i= 0;
		/* Read each device line */
		while(fgets(buff, sizeof(buff), fh))
		{
			char name[IFNAMSIZ + 1];
			char *s;

			/* Skip empty or almost empty lines. It seems that in some
			* cases fgets return a line with only a newline. */
			if((buff[0] == '\0') || (buff[1] == '\0'))
				continue;

			/* Extract interface name */
			s = iw_get_ifname_for_boa(name, sizeof(name), buff);

			if(!s)
			{
				/* Failed to parse, complain and continue */
				fprintf(stderr, "Cannot parse " PROC_NET_WIRELESS "\n");


			}
			else
			{
			    /* Got it, print info about this interface */
				if (strstr(name, "ra") != NULL)
				{
					{
						strcpy((char *)dev_info[i].Name, name);
						
						// get mac addr
						ret = OidQueryInformation(OID_802_3_CURRENT_ADDRESS, socket, (char *)dev_info[i].Name, &CurrentAddress, sizeof(CurrentAddress));

						if (ret >= 0)
						{
							sprintf((char *)dev_info[i].Mac, "%02X:%02X:%02X:%02X:%02X:%02X", CurrentAddress[0], CurrentAddress[1],
							CurrentAddress[2], CurrentAddress[3], CurrentAddress[4], CurrentAddress[5]);
						}
						else
							sprintf((char *)dev_info[i].Mac, "00:00:00:00:00:00");

						i++;
					}
				}
			}

		}
		num_of_dev = i;
		fclose(fh);
    }
	else
    {
		/* Get list of configured devices using "traditional" way */
		ifc.ifc_len = sizeof(buff);
		ifc.ifc_buf = buff;
		if(ioctl(socket, SIOCGIFCONF, &ifc) < 0)
		{
			fprintf(stderr, "SIOCGIFCONF: %s\n", strerror(errno));
			return;
		}
		ifr = ifc.ifc_req;

		/* Print them */
		for(i = ifc.ifc_len / sizeof(struct ifreq); --i >= 0; ifr++)
			strcpy((char *)dev_info[i].Name, ifr->ifr_name);
    }
}

//get init value of advance page
void getInitValueOfAdvancePage()
{
	char tmpString[8];
	int s, country_region_a=0, country_region_bg=0;
	int bg_protection=0, tx_burst=0, short_slot_time=0, wireless_mode=0;
#ifdef PHY_N_SUPPORT
	int ht_mode=0, ht_bw=0, ht_gi=0, ht_mcs=0; //11n, kathy
#endif
	unsigned long country_region=0;
	RT_802_11_STA_CONFIG	configStation;

	// for framework
	UCHAR	country_code[3];
	UCHAR	channel_geography;

	bzero(tmpString, sizeof(tmpString));
	s = socket(AF_INET, SOCK_DGRAM, 0);

#ifdef PHY_N_SUPPORT
	// query 11n ht phy mode
	{
		OID_SET_HT_PHYMODE phymode;
		
		if (OidQueryInformation(RT_OID_802_11_QUERY_DAT_HT_PHYMODE, s, (char *)dev_info[dev_idx].Name, &phymode, sizeof(phymode))>=0 )
		{
			wireless_mode = phymode.PhyMode;
			ht_mode = phymode.HtMode;
			ht_bw = phymode.BW;
			ht_gi = phymode.SHORTGI;
			ht_mcs = phymode.MCS;


			sprintf(tmpString, "%d", wireless_mode);
			nvram_set("staWirelessMode", tmpString);

			sprintf(tmpString, "%d", ht_mode);
			nvram_set("staHT_OpMode", tmpString);

			sprintf(tmpString, "%d", ht_bw);
			nvram_set("staHT_BW", tmpString);

			sprintf(tmpString, "%d", ht_gi);
			nvram_set("staHT_GI", tmpString);

			sprintf(tmpString, "%d", ht_mcs);
			nvram_set("staHT_MCS", tmpString);
		}
		else
			fprintf(stderr, "Query RT_OID_802_11_QUERY_DAT_HT_PHYMODE error! dev_name=%s\n", dev_info[dev_idx].Name);
	}
#else
	//query wireless mode
	OidQueryInformation(RT_OID_802_11_PHY_MODE, s, (char *)dev_info[dev_idx].Name, &wireless_mode, sizeof(wireless_mode));
	sprintf(tmpString, "%d", wireless_mode);
	nvram_set("staWirelessMode", tmpString);
#endif


	//query country region
	OidQueryInformation(RT_OID_802_11_COUNTRY_REGION, s, (char *)dev_info[dev_idx].Name, &country_region, sizeof(country_region));
	country_region_bg = country_region & 0x7f;
	country_region_a = (country_region & 0x7f00) >> 8;
	sprintf(tmpString, "%d", country_region_bg);
	nvram_set("staCountryRegionBG", tmpString);

	sprintf(tmpString, "%d", country_region_a);
	nvram_set("staCountryRegionA", tmpString);

	OidQueryInformation(RT_OID_802_11_STA_CONFIG, s, (char *)dev_info[dev_idx].Name, &configStation, sizeof(configStation));
	
	tx_burst = configStation.EnableTxBurst;
	bg_protection = configStation.UseBGProtection;
	short_slot_time = configStation.UseShortSlotTime;

	sprintf(tmpString, "%d", bg_protection);
	nvram_set("staBGProtection", tmpString);

	sprintf(tmpString, "%d", tx_burst);
	nvram_set("staTxBurst", tmpString);

	sprintf(tmpString, "%d", short_slot_time);
	nvram_set("staShortSlotTime", tmpString);

	// for framework
	memset(country_code, 0x00, sizeof(country_code));
	//memset(channel_geography, 0x00, sizeof(channel_geography));
	OidQueryInformation(OID_802_11_GET_COUNTRY_CODE, s, (char *)dev_info[dev_idx].Name, &country_code, sizeof(country_code));
	nvram_set("staCountryCode", (char *)country_code);

	OidQueryInformation(OID_802_11_GET_CHANNEL_GEOGRAPHY, s, (char *)dev_info[dev_idx].Name, &channel_geography, sizeof(channel_geography));
	memset(tmpString, 0x00, sizeof(tmpString));
	sprintf(tmpString, "%d", channel_geography);
	nvram_set("staChannelGeography", tmpString);

	close(s);
}

#ifdef QOS_DLS_SUPPORT
	// query DLS and WMM value from driver.
void getInitValueOfWMMPage()
{
	int s;
	char wmm_enable=0, wmm_ps_enable=0, wmm_ps_mode_acbk=0, wmm_ps_mode_acbe=0, wmm_ps_mode_acvi=0, wmm_ps_mode_acvo=0 ,wmm_dls_enable=0;
	unsigned long apsd;
	char tmp_str[4];
	
	s = socket(AF_INET, SOCK_DGRAM, 0);
	OidQueryInformation(RT_OID_802_11_QUERY_WMM, s, (char *)dev_info[dev_idx].Name, &wmm_enable, sizeof(wmm_enable));
	OidQueryInformation(RT_OID_802_11_QUERY_APSD_SETTING, s, (char *)dev_info[dev_idx].Name, &apsd, sizeof(apsd));
	OidQueryInformation(RT_OID_802_11_QUERY_DLS, s, (char *)dev_info[dev_idx].Name, &wmm_dls_enable, sizeof(wmm_dls_enable));

	if ( wmm_enable )
	{
		if ( (apsd & 0x00000001) == 1)
		{
			wmm_ps_enable = 1;

			if ( (apsd & 0x00000002) == 1)
				 wmm_ps_mode_acbk = 1;

			if ( (apsd & 0x00000004) == 1 )
				wmm_ps_mode_acbe = 1;

			if ( (apsd & 0x00000008) == 1)
				wmm_ps_mode_acvi = 1;

			if ( (apsd & 0x00000010) == 1)
				wmm_ps_mode_acvo = 1;
		}
		else
			wmm_ps_enable = 1;
	}
	close(s);
	sprintf(tmp_str, "%d", wmm_enable);
	nvram_set("staWMM", tmp_str);

	sprintf(tmp_str, "%d", wmm_ps_enable);
	nvram_set("staAPSD", tmp_str);

	sprintf(tmp_str, "%d", wmm_dls_enable);
	nvram_set("staDLS", tmp_str);

	sprintf(tmp_str, "%d", wmm_ps_enable);
	nvram_set("staAPSD", tmp_str);

	sprintf(tmp_str, "%d", wmm_ps_mode_acbk);
	nvram_set("staPSACBK", tmp_str);

	sprintf(tmp_str, "%d", wmm_ps_mode_acbe);
	nvram_set("staPSACBE", tmp_str);

	sprintf(tmp_str, "%d", wmm_ps_mode_acvi);
	nvram_set("staPSACVI", tmp_str);

	sprintf(tmp_str, "%d", wmm_ps_mode_acvo);
	nvram_set("staPSACVO", tmp_str);
}
#endif



void findRT28xxCardDat(int multicard_support, int skt)
{
	if (multicard_support)
	{
		char device_name[12];
		char tmpbuf[128];
		FILE *fh;
		

		memset(device_name, 0x00, sizeof(device_name));
		if (OidQueryInformation(RT_OID_DEVICE_NAME, skt, (char *)dev_info[dev_idx].Name, &device_name, sizeof(dev_info[dev_idx].Name)) < 0 )
		{
			fprintf(stderr, "Query RT_OID_DEVICE_NAME error!\n");
			return;
		}
		sprintf(tmpbuf, "%s%s/%sCard.dat", DAT_PATH, device_name,device_name);

		fh = fopen(tmpbuf, "r");

		if (fh == NULL)
		{

			fh = fopen(tmpbuf, "w+");
			int idx=0;
			// to create a new RT28xxCard.dat
			
			fprintf(fh, "#The word of \"Default\" must not be removed, maximum 32 cards, 00 ~ 31\n");
			fprintf(fh, "Default\n");
			fprintf(fh, "\n");
			fprintf(fh, "#CARDID, MAC, CARDTYPE\n");
			fprintf(fh, "SELECT=MAC\n");
			fprintf(fh, "\n");
			
			do
			{
				fprintf(fh, "%02xMAC%s=/etc/Wireless/%s/%s%d.dat\n", idx, dev_info[idx].Mac, device_name, device_name, idx+1);
				sprintf(tmpbuf, "mv %s%s/%s.dat %s%s/%s%d.dat ", DAT_PATH, device_name, device_name, DAT_PATH, device_name, device_name, idx+1);
				system(tmpbuf);
				idx++;
			}while (strlen((char *)dev_info[idx].Mac)>0);
			
			fclose(fh);

			reloadWirelessModule();
		}
		else
		{			
			fclose(fh);
		}
	}
	else
		return;

}

