/*
 *  Boa, an http server
 *  Copyright (C) 1995 Paul Phillips <paulp@go2net.com>
 *  Some changes Copyright (C) 1996,97 Larry Doolittle <ldoolitt@jlab.org>
 *  Some changes Copyright (C) 1997 Jon Nelson <jnelson@boa.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* $Id: globals.h,v 1.2 2008/05/19 11:39:41 kathy Exp $*/

#ifndef _GLOBALS_H
#define _GLOBALS_H

#ifndef IFNAMSIZ
#define IFNAMSIZ 16
#endif

#include "rtmp_type.h"
#include "oid.h"

/********************** METHODS **********************/
enum HTTP_METHOD { M_GET = 1, M_HEAD, M_PUT, M_POST,
    M_DELETE, M_LINK, M_UNLINK, M_MOVE, M_TRACE
};

/******************* HTTP VERSIONS *******************/
enum HTTP_VERSION { HTTP09=1, HTTP10, HTTP11 };

/************** REQUEST STATUS (req->status) ***************/
enum REQ_STATUS { READ_HEADER, ONE_CR, ONE_LF, TWO_CR,
    BODY_READ, BODY_WRITE,
    WRITE,
    PIPE_READ, PIPE_WRITE,
    IOSHUFFLE,
    DONE,
    TIMED_OUT,
    DEAD
};

/******************* RESPONSE CODES ******************/
enum RESPONSE_CODE { R_CONTINUE = 100,
                     R_REQUEST_OK = 200,
                     R_CREATED,
                     R_ACCEPTED,
                     R_PROVISIONAL,
                     R_NO_CONTENT,
                     R_205,
                     R_PARTIAL_CONTENT,
                     R_MULTIPLE = 300,
                     R_MOVED_PERM,
                     R_MOVED_TEMP,
                     R_303,
                     R_NOT_MODIFIED,
                     R_BAD_REQUEST = 400,
                     R_UNAUTHORIZED,
                     R_PAYMENT,
                     R_FORBIDDEN,
                     R_NOT_FOUND,
                     R_METHOD_NA, /* method not allowed */
                     R_NON_ACC,   /* non acceptable */
                     R_PROXY,     /* proxy auth required */
                     R_REQUEST_TO, /* request timeout */
                     R_CONFLICT,
                     R_GONE,
                     R_LENGTH_REQUIRED,
                     R_PRECONDITION_FAILED,
                     R_REQUEST_URI_TOO_LONG = 414,
                     R_INVALID_RANGE = 416,
                     R_ERROR = 500,
                     R_NOT_IMP,
                     R_BAD_GATEWAY,
                     R_SERVICE_UNAV,
                     R_GATEWAY_TO, /* gateway timeout */
                     R_BAD_VERSION };

/************* ALIAS TYPES (aliasp->type) ***************/
enum ALIAS { ALIAS, SCRIPTALIAS, REDIRECT };

/*********** KEEPALIVE CONSTANTS (req->keepalive) *******/
enum KA_STATUS { KA_INACTIVE, KA_ACTIVE, KA_STOPPED };

/********* CGI STATUS CONSTANTS (req->cgi_status) *******/
enum CGI_STATUS { CGI_PARSE, CGI_BUFFER, CGI_DONE };

/************** CGI TYPE (req->is_cgi) ******************/
enum CGI_TYPE { NPH = 1, CGI };

/**************** STRUCTURES ****************************/
struct range {
    unsigned long start;
    unsigned long stop;
    struct range *next;
};

typedef struct range Range;

struct mmap_entry {
    dev_t dev;
    ino_t ino;
    char *mmap;
    int use_count;
    off_t len;
};

struct request {                /* pending requests */
    enum REQ_STATUS status;
    enum KA_STATUS keepalive;   /* keepalive status */
    enum HTTP_VERSION http_version;
    enum HTTP_METHOD method;    /* M_GET, M_POST, etc. */
    enum RESPONSE_CODE response_status; /* R_NOT_FOUND, etc.. */

    enum CGI_TYPE cgi_type;
    enum CGI_STATUS cgi_status;

    /* should pollfd_id be zeroable or no ? */
#ifdef HAVE_POLL
    int pollfd_id;
#endif

    char *pathname;             /* pathname of requested file */

    Range *ranges;              /* our Ranges */
    int numranges;

    int data_fd;                /* fd of data */
    unsigned long filesize;     /* filesize */
    unsigned long filepos;      /* position in file */
    unsigned long bytes_written; /* total bytes written (sans header) */
    char *data_mem;             /* mmapped/malloced char array */

#ifdef USE_AUTH
	char *authorization;
#endif

    char *logline;              /* line to log file */

    char *header_line;          /* beginning of un or incompletely processed header line */
    char *header_end;           /* last known end of header, or end of processed data */
    int parse_pos;              /* how much have we parsed */

    int buffer_start;           /* where the buffer starts */
    int buffer_end;             /* where the buffer ends */

    char *if_modified_since;    /* If-Modified-Since */
    time_t last_modified;       /* Last-modified: */

    /* CGI vars */
    int cgi_env_index;          /* index into array */

    /* Agent and referer for logfiles */
    char *header_host;
    char *header_user_agent;
    char *header_referer;
    char *header_ifrange;
    char *host;                 /* what we end up using for 'host', no matter the contents of header_host */

    int post_data_fd;           /* fd for post data tmpfile */

    char *path_info;            /* env variable */
    char *path_translated;      /* env variable */
    char *script_name;          /* env variable */
    char *query_string;         /* env variable */
    char *content_type;         /* env variable */
    char *content_length;       /* env variable */

    struct mmap_entry *mmap_entry_var;

    /* everything **above** this line is zeroed in sanitize_request */
    /* this may include 'fd' */
    /* in sanitize_request with the 'new' parameter set to 1,
     * kacount is set to ka_max and client_stream_pos is also zeroed.
     * Also, time_last is set to 'NOW'
     */
    int fd;                     /* client's socket fd */
    time_t time_last;           /* time of last succ. op. */
    char local_ip_addr[BOA_NI_MAXHOST]; /* for virtualhost */
    char remote_ip_addr[BOA_NI_MAXHOST]; /* after inet_ntoa */
    unsigned int remote_port;            /* could be used for ident */

    unsigned int kacount;                /* keepalive count */
    int client_stream_pos;      /* how much have we read... */

    char user[16];			/* user's login name */

    /* everything below this line is kept regardless */
    char buffer[BUFFER_SIZE + 1]; /* generic I/O buffer */
    char request_uri[MAX_HEADER_LENGTH + 1]; /* uri */
    char client_stream[CLIENT_STREAM_SIZE]; /* data from client - fit or be hosed */
    char *cgi_env[CGI_ENV_MAX + 4]; /* CGI environment */

#ifdef ACCEPT_ON
    char accept[MAX_ACCEPT_LENGTH]; /* Accept: fields */
#endif

    struct request *next;       /* next */
    struct request *prev;       /* previous */
};

typedef struct request request;

struct status {
    long requests;
    long errors;
};

/* 2004.12.21 add by arvin tai */
#define MAX_SSID_NUMBER		4
typedef void cgi_handler(char *data, FILE *out);
typedef void ssi_handler( FILE *out, char *ssi_arg);
typedef int multipart_handler(FILE *stream, int len, int len_boundary);

typedef struct 
{
    const char *name;      /* The name of the script as given in the URL */
    cgi_handler *handler;  /* The function which should be called */
    multipart_handler *multi_handler;  /* The function which should be called */
} CGI_ENTRY;

/* add by Dennis Lee to add the SSI Control*/

typedef struct 
{
    const char *name;      /* The name of the script as given in the URL */
    ssi_handler *handler;  /* The function which should be called */
} SSI_ENTRY;

struct name_val {
    char * name;
    char * value;
};

struct httpform {
    int nv_ct;    /* number of entry in name value pairs */
    struct name_val nameval[1];
};

struct www_name {
    char name[100];
};

struct www_file {
    int f_cnt;
    struct www_name f_name[1];
};

typedef struct 
{
    const char *suffix;  /* Suffix of filename, excluding '.', e.g. "txt" */ 
    const char *type;    /* Content-type, e.g. "text/plain" */
} FILETYPE;
typedef struct site_survey_tbl {
	unsigned char      macAddr[20];
	char		       ssid[33];
	char 		       auth[8];
	char               channel[4];
	char               wirelessmode[4];
	unsigned char      rssi;
}site_survey_tbl;

typedef struct multi_ssid_s {
	int  enable;
    char SSID[33];
    int  SecurityMode;
    char AuthMode[14];
    char EncrypType[8];
    char WPAPSK[65];
    int  DefaultKeyID;
    int  Key1Type;
    char Key1Str[27];
    int  Key2Type;
    char Key2Str[27];
    int  Key3Type;
    char Key3Str[27];
    int  Key4Type;
    char Key4Str[27];
    int  IEEE8021X;
    int  TxRate;
    int  HideSSID;
    int  PreAuth;
    char VlanName[20];
    int  VlanId;
    int  VlanPriority;
}multi_ssid_s;

typedef struct web_chantbl
{
    char channel[30];
} web_chantbl;

extern int  reboot_flag;
extern int  redirectSetupPage;
extern int  ATE_ACTION_MODE;
extern int  ATE_ACTION_START;
extern char ateda[];
extern char atesa[];
extern char atebssid[];
extern char atetxpow;
extern int  atechannel;
extern char atetxfreqoffset;
extern int  atetxlen;
extern long atetxcnt;
extern int  atetxrate;
extern int  eeprom_read_write;
extern int  eeprom_offset;
extern char eeprom_value[];
extern int  mac_read_write;
extern int  mac_offset;
extern char mac_value[];
extern int  bbp_read_write;
extern int  bbp_offset;
extern char bbp_value[];
/* end */

extern struct status status;

extern char *optarg;            /* For getopt */

extern request *request_ready;  /* first in ready list */
extern request *request_block;  /* first in blocked list */
extern request *request_free;   /* first in free list */

#ifdef HAVE_POLL
extern struct pollfd *pfds;
extern unsigned int pfd_len;
#else
extern fd_set block_read_fdset; /* fds blocked on read */
extern fd_set block_write_fdset; /* fds blocked on write */
extern int max_fd;
#endif

/* global server variables */

extern char *access_log_name;
extern char *error_log_name;
extern char *cgi_log_name;
extern int cgi_log_fd;
extern int use_localtime;

extern unsigned int user_server_port; // assign vale from user mode
extern unsigned int server_port;
extern uid_t server_uid;
extern gid_t server_gid;
extern char *server_admin;
extern char *server_root;
extern char *server_name;
extern char *server_ip;

extern char *user_document_root;
extern char *document_root;
extern char *user_dir;
extern char *directory_index;
extern char *default_type;
extern char *default_charset;
extern char *dirmaker;
extern char *mime_types;
extern char *pid_file;
extern char *cachedir;

extern const char *tempdir;

extern char *cgi_path;
extern short common_cgi_env_count;
extern int single_post_limit;
extern int conceal_server_identity;

extern int unsigned ka_timeout;
extern int unsigned default_timeout;
extern int unsigned ka_max;

extern int sighup_flag;
extern int sigchld_flag;
extern int sigalrm_flag;
extern int sigterm_flag;
extern time_t start_time;

extern int pending_requests;
extern unsigned max_connections;

extern int verbose_cgi_logs;

extern int backlog;
extern time_t current_time;

extern int virtualhost;
extern char *vhost_root;
extern const char *default_vhost;

extern unsigned total_connections;
extern unsigned int system_bufsize;      /* Default size of SNDBUF given by system */

extern sigjmp_buf env;
extern int handle_sigbus;
extern unsigned int cgi_umask;


//kathy, 2006-0810
char *http_boundary;
int  firmware_fd;
int  total_count;

#define IDS_GENERAL_LINK_UP		"Link is Up"
#define IDS_GENERAL_LINK_DOWN	"Link is Down"
#define IDS_HW_RADIO_OFF		"Hardware radio off"
#define IDS_SW_RADIO_OFF		"Software radio off"
#define IDS_AUTH_FAIL			"Open authentication fail"
#define IDS_AUTH_FAIL_KEYS		"Shared authentication fail"
#define IDS_ASSOC_FAIL			"Association failed"
#define IDS_EAP_MIC_FAILURE		"Deauthencation because MIC failure"
#define IDS_EAP_4WAY_TIMEOUT		"Deauthencation on 4-way handshake timeout"
#define IDS_EAP_GROUP_KEY_TIMEOUT	"Deauthencation on group key handshake timeout"
#define IDS_EAP_SUCCESS			"EAP successd"

#define IDS_RADIOOFF			"RF OFF"
#define IDS_DISCONNECTED		"Disconnected"

#define IDS_LINK_GOOD			"Good"
#define IDS_LINK_NORMAL			"Normal"
#define IDS_LINK_WEAK			"Weak"

typedef struct _EXTRA_INFO_DATA
{
	ULONG		nExtraInfoDef;
	char*		nExtraInfoString;
} EXTRA_INFO_DATA, *PEXTRA_INFO_DATA;

//copy from RT61 Utility

typedef struct _PAIR_CHANNEL_FREQ_ENTRY
{
	unsigned long	lChannel;
	unsigned long	lFreq;
} PAIR_CHANNEL_FREQ_ENTRY, *PPAIR_CHANNEL_FREQ_ENTRY;

//UI struct , kathy
// wpa_supplicant.conf
#define IDENTITY_LENGTH	512
#define CERT_PATH_LENGTH	64
#define PRIVATE_KEY_PATH_LENGTH	64


typedef enum _RT_WPA_SUPPLICANT_KEY_MGMT {
	Rtwpa_supplicantKeyMgmtWPANONE,
	Rtwpa_supplicantKeyMgmtWPAPSK,
	Rtwpa_supplicantKeyMgmtWPAEAP,
	Rtwpa_supplicantKeyMgmtIEEE8021X,
	Rtwpa_supplicantKeyMgmtNONE
} RT_WPA_SUPPLICANT_KEY_MGMT, *PRT_WPA_SUPPLICANT_KEY_MGMT;

typedef enum _RT_WPA_SUPPLICANT_EAP {
	Rtwpa_supplicantEAPMD5,
	Rtwpa_supplicantEAPMSCHAPV2,
	Rtwpa_supplicantEAPOTP,
	Rtwpa_supplicantEAPGTC,
	Rtwpa_supplicantEAPTLS,
	Rtwpa_supplicantEAPPEAP,
	Rtwpa_supplicantEAPTTLS,
	Rtwpa_supplicantEAPNONE
} RT_WPA_SUPPLICANT_EAP, *PRT_WPA_SUPPLICANT_EAP;

typedef enum _RT_WPA_SUPPLICANT_EAPOL_FLAGS {
	Rtwpa_supplicantEAPOLFLAGSSTATICWEPKEY,
	Rtwpa_supplicantEAPOLFLAGSDYNAMICALUNICASTWEPKEY,
	Rtwpa_supplicantEAPOLFLAGSDYNAMICALBROADCASTWEPKEY,
	Rtwpa_supplicantEAPOLFLAGSDYNAMICALWEPKEY,
	Rtwpa_supplicantEAPOLFLAGSDEFAULT = Rtwpa_supplicantEAPOLFLAGSDYNAMICALWEPKEY
} _RT_WPA_SUPPLICANT_EAPOL_FLAGS, *P_RT_WPA_SUPPLICANT_EAPOL_FLAGS;

typedef enum _RT_WPA_SUPPLICANT_GROUP {
	Rtwpa_supplicantGROUPCCMP,
	Rtwpa_supplicantGROUPTKIP,
	Rtwpa_supplicantGROUPWEP104,
	Rtwpa_supplicantGROUPWEP40,
	Rtwpa_supplicantGROUPDEFAULT  // CCMP, TKIP, WEP104, WEP40
} RT_WPA_SUPPLICANT_GROUP, *PRT_WPA_SUPPLICANT_GROUP;

typedef enum _RT_WPA_SUPPLICANT_PAIRWISE {
	Rtwpa_supplicantPAIRWISECCMP,
	Rtwpa_supplicantPAIRWISETKIP,
	Rtwpa_supplicantPAIRWISENONE,
	Rtwpa_supplicantPAIRWISEDEFAULT
} RT_WPA_SUPPLICANT_PAIRWISE, *PRT_WPA_SUPPLICANT_PAIRWISE;

typedef enum _RT_WPA_SUPPLICANT_PROTO {
	Rtwpa_supplicantPROTOWPA,	//WPA
	Rtwpa_supplicantPROTORSN,  //WPA2
	Rtwpa_supplicantPROTODEFAULT	//WPA RSN
} RT_WPA_SUPPLICANT_PROTO, *PRT_WPA_SUPPLICANT_PROTO;

typedef enum _RT_WPA_SUPPLICANT_TUNNEL {
	Rtwpa_supplicantTUNNELMSCHAP,
	Rtwpa_supplicantTUNNELMSCHAPV2,
	Rtwpa_supplicantTUNNELPAP,
	Rtwpa_supplicantTUNNENONE
} RT_WPA_SUPPLICANT_TUNNEL, *PRT_WPA_SUPPLICANT_TUNNEL;
//wpa_supplicant_support
typedef struct _RT_PROFILE_SETTING {
    UINT 							ProfileDataType; //0x18140201
    UCHAR							Profile[32+1];
    UCHAR							SSID[NDIS_802_11_LENGTH_SSID+1];
    UINT							SsidLen;
    UINT							Channel;
    NDIS_802_11_AUTHENTICATION_MODE	Authentication; //Ndis802_11AuthModeOpen, Ndis802_11AuthModeShared
                                                                //Ndis802_11AuthModeWPAPSK
    NDIS_802_11_WEP_STATUS			Encryption; //Ndis802_11WEPEnabled, Ndis802_11WEPDisabled
                                                            //Ndis802_11Encryption2Enabled, Ndis802_11Encryption3Enabled
    NDIS_802_11_NETWORK_INFRASTRUCTURE	NetworkType;
    UINT							KeyDefaultId;
    UINT							Key1Type;
    UINT							Key2Type;
    UINT							Key3Type;
    UINT							Key4Type;
    UINT							Key1Length;
    UINT							Key2Length;
    UINT							Key3Length;
    UINT							Key4Length;
    CHAR							Key1[26+1];
    CHAR							Key2[26+1];
    CHAR							Key3[26+1];
    CHAR							Key4[26+1];
    CHAR							WpaPsk[64+1];//[32+1];
    UINT							TransRate;
    UINT							TransPower;
    BOOLEAN							RTSCheck;  //boolean
    UINT 							RTS;
    BOOLEAN								FragmentCheck; //boolean
    UINT							Fragment;
    NDIS_802_11_POWER_MODE			PSmode;
    RT_802_11_PREAMBLE				PreamType;
    UINT							AntennaRx;
    UINT							AntennaTx;
    UINT							CountryRegion;
    //Advance					
    //RT_802_11_TX_RATES				ConfigSta;
    UINT							AdhocMode;
    //UCHAR							reserved[64];
    UINT							Active; // 0 is the profile is set as connection profile, 1 is not. add by kathy, 2006-1023
//#ifdef WPA_SUPPLICANT_SUPPORT  

	RT_WPA_SUPPLICANT_KEY_MGMT		KeyMgmt;
	RT_WPA_SUPPLICANT_EAP			EAP;
	UCHAR							Identity[IDENTITY_LENGTH];
	UCHAR							Password[32];
	UCHAR							CACert[CERT_PATH_LENGTH];
	UCHAR							ClientCert[CERT_PATH_LENGTH];
	UCHAR							PrivateKey[PRIVATE_KEY_PATH_LENGTH];
	UCHAR							PrivateKeyPassword[32];
	RT_WPA_SUPPLICANT_TUNNEL		Tunnel;
//#endif

    struct  _RT_PROFILE_SETTING		*Next;
} RT_PROFILE_SETTING, *PRT_PROFILE_SETTING;

PRT_PROFILE_SETTING	selectedProfileSetting, headerProfileSetting, currentProfileSetting;


//Device Card Info
typedef struct _RT_DEVICE_CARD_INFO {
	UCHAR Name[16];
	UCHAR Mac[20];
	UCHAR Flag;
} RT_DEVICE_CARD_INFO, *PRT_DEVICE_CARD_INFO;

#endif
