/*
 *  Boa, an http server
 *  Copyright (C) 1995 Paul Phillips <paulp@go2net.com>
 *  Some changes Copyright (C) 1996,97 Larry Doolittle <ldoolitt@boa.org>
 *  Some changes Copyright (C) 1996 Charles F. Randall <crandall@goldsys.com>
 *  Some changes Copyright (C) 1996-99,2003 Jon Nelson <jnelson@boa.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/* $Id: util.c,v 1.1 2008/04/23 06:38:46 kathy Exp $ */

#include <sys/ioctl.h>
#include <net/if.h>
#include <net/route.h>
#include <arpa/inet.h>
#include <net/if_arp.h>
#include "boa.h"
#include "nvram.h"

static int date_to_tm(struct tm *file_gmt, const char *cmtime);

/* Don't need or want the trailing nul for these character arrays */
static const char month_tab[48] =
    "Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec ";
static const char day_tab[28] = "Sun,Mon,Tue,Wed,Thu,Fri,Sat,";

/* add by arvin tai 2004.12.21 */
int skfd;
short int g_flags;
char g_addr[16], g_net[16], g_brd[16], g_hw[18];
/* add end */

/*
 * Name: clean_pathname
 *
 * Description: Replaces unsafe/incorrect instances of:
 *  //[...] with /
 *  /./ with /
 *  /../ with / (technically not what we want, but browsers should deal
 *   with this, not servers)
 */

void clean_pathname(char *pathname)
{
    char *cleanpath, c;

    cleanpath = pathname;
    while ((c = *pathname++)) {
        if (c == '/') {
            while (1) {
                if (*pathname == '/')
                    pathname++;
                else if (*pathname == '.' && *(pathname + 1) == '/')
                    pathname += 2;
                else if (*pathname == '.' && *(pathname + 1) == '.' &&
                         *(pathname + 2) == '/') {
                    pathname += 3;
                } else
                    break;
            }
            c = '/';
        }
        *cleanpath++ = c;
    }

    *cleanpath = '\0';
}

#if 0
char *new_clean_pathname(char *pathname)
{
    static char *segment[50];
    int seg_count = 0;
    char *a, *cleanpath, c;

    a = pathname;
    segment[seg_count] = pathname;
    cleanpath = pathname;

    while ((c = *pathname++)) {
        if (c == '/') {         /* /?? */
            while (1) {         /* everything in this loop gets eliminated */
                if (*pathname == '/') /* // */
                    pathname++;
                else if (*pathname == '.') { /* /. */
                    if (*(pathname + 1) == '/') /* /./ */
                        pathname += 2;
                    else if (*(pathname + 1) == '\0') /* /.$ */
                        pathname += 1;
                    else if (*(pathname + 1) == '.') { /* /.. */
                        if (*(pathname + 2) == '/') /* /../ */
                            pathname += 3;
                        else if (*(pathname + 1) == '\0') /* /..$ */
                            pathname += 2;
                        /*
                           cleanpath goes *back* one
                         */
                        if (seg_count)
                            cleanpath = segment[--seg_count];
                        else {
                            *a = '\0';
                            return a;
                        }
                    } else {    /* /.blah */
                        break;
                    }
                } else {        /* we have /something */
                    break;
                }
            }
            if (seg_count > 49) { /* we can store in spots 0...49 */
                *a = '\0';
                return a;
            }
            *cleanpath = '/';
            segment[seg_count++] = cleanpath++;
        } else
            *cleanpath++ = c;
    }

    *cleanpath = '\0';
    return a;
}
#endif

/*
 * Name: get_commonlog_time
 *
 * Description: Returns the current time in common log format in a static
 * char buffer.
 *
 * commonlog time is exactly 25 characters long
 * because this is only used in logging, we add " [" before and "] " after
 * making 29 characters
 * "[27/Feb/1998:20:20:04 +0000] "
 *
 * Contrast with rfc822 time:
 * "Sun, 06 Nov 1994 08:49:37 GMT"
 *
 * Altered 10 Jan 2000 by Jon Nelson ala Drew Streib for non UTC logging
 *
 */

char *get_commonlog_time(void)
{
    struct tm *t;
    char *p;
    unsigned int a;
    static char buf[30];
    int time_offset;

    if (use_localtime) {
        t = localtime(&current_time);
        time_offset = TIMEZONE_OFFSET(t);
    } else {
        t = gmtime(&current_time);
        time_offset = 0;
    }

    p = buf + 29;
    *p-- = '\0';
    *p-- = ' ';
    *p-- = ']';
    a = abs(time_offset / 60);
    *p-- = '0' + a % 10;
    a /= 10;
    *p-- = '0' + a % 6;
    a /= 6;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p-- = (time_offset >= 0) ? '+' : '-';
    *p-- = ' ';

    a = t->tm_sec;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p-- = ':';
    a = t->tm_min;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p-- = ':';
    a = t->tm_hour;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p-- = ':';
    a = 1900 + t->tm_year;
    while (a) {
        *p-- = '0' + a % 10;
        a /= 10;
    }
    /* p points to an unused spot */
    *p-- = '/';
    p -= 2;
    memcpy(p--, month_tab + 4 * (t->tm_mon), 3);
    *p-- = '/';
    a = t->tm_mday;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p = '[';
    return p;                   /* should be same as returning buf */
}

/*
 * Name: month2int
 *
 * Description: Turns a three letter month into a 0-11 int
 *
 * Note: This function is from wn-v1.07 -- it's clever and fast
 */

int month2int(const char *monthname)
{
    switch (*monthname) {
    case 'A':
        return (*++monthname == 'p' ? 3 : 7);
    case 'D':
        return (11);
    case 'F':
        return (1);
    case 'J':
        if (*++monthname == 'a')
            return 0;
        return (*++monthname == 'n' ? 5 : 6);
    case 'M':
        return (*(monthname + 2) == 'r' ? 2 : 4);
    case 'N':
        return (10);
    case 'O':
        return (9);
    case 'S':
        return (8);
    default:
        return (-1);
    }
}

/*
 * Name: date_to_seconds
 * Description:
 *

 Sun, 06 Nov 1994 08:49:37 GMT    ; RFC 822, updated by RFC 1123
 Sunday, 06-Nov-94 08:49:37 GMT   ; RFC 850, obsoleted by RFC 1036
 Sun Nov  6 08:49:37 1994         ; ANSI C's asctime() format
 31 September 2000 23:59:59 GMT   ; non-standard

 * RETURN VALUES:
 *  -1 for error, 0 for success
 */

static int date_to_tm(struct tm *parsed_gmt, const char *cmtime)
{
    char monthname[10 + 1];
    const char *cmtime_start = cmtime;
    int day, year, hour, minute, second;

    /* we don't use the weekday portion, so skip over it */
    while (*cmtime != ' ' && *cmtime != '\0')
        ++cmtime;

    if (*cmtime != ' ')
        return -1;
    /* the pre-space in the third scanf skips whitespace for the string */
    if (sscanf(cmtime, "%d %3s %d %d:%d:%d GMT", /* RFC 1123 */
               &day, monthname, &year, &hour, &minute, &second) == 6) {
    } else if (sscanf(cmtime, "%d-%3s-%d %d:%d:%d GMT", /* RFC 1036 */
                      &day, monthname, &year, &hour, &minute, &second) == 6) {
    } else if (sscanf(cmtime, "%3s %d %d:%d:%d %d", /* asctime() format */
                      monthname, &day, &hour, &minute, &second, &year) == 6) {
        /*
         *  allow this non-standard date format: 31 September 2000 23:59:59 GMT
         * NOTE: Use 'cmtime_start' instead of 'cmtime' here, because the date *starts*
         *       with the day, versus a throwaway item
         */
    } else if (sscanf(cmtime_start, "%d %10s %d %d:%d:%d GMT",
                      &day, monthname, &year, &hour, &minute, &second) == 6) {
    } else {
        log_error_time();
        fprintf(stderr,
                "Error in %s, line %d: Unable to sscanf \"%s\"\n",
                __FILE__, __LINE__, cmtime);
        return -1;              /* error */
    }

    if (year < 70)
        year += 100;
    if (year > 1900)
        year -= 1900;

    parsed_gmt->tm_sec = second;
    parsed_gmt->tm_min = minute;
    parsed_gmt->tm_hour = hour;
    parsed_gmt->tm_mday = day;
    parsed_gmt->tm_mon = month2int(monthname);
    parsed_gmt->tm_year = year;
    parsed_gmt->tm_wday = 0;
    parsed_gmt->tm_yday = 0;
    parsed_gmt->tm_isdst = 0;

    if (parsed_gmt->tm_mon == -1) {
        log_error_time();
        fprintf(stderr, "Invalid month name: \"%s\"\n", monthname);
        return -1;
    }

    /* adapted from Squid 2.5 */
    if (parsed_gmt->tm_sec < 0 || parsed_gmt->tm_sec > 59)
        return -1;
    if (parsed_gmt->tm_min < 0 || parsed_gmt->tm_min > 59)
        return -1;
    if (parsed_gmt->tm_hour < 0 || parsed_gmt->tm_hour > 23)
        return -1;
    if (parsed_gmt->tm_mday < 1 || parsed_gmt->tm_mday > 31)
        return -1;
    if (parsed_gmt->tm_mon < 0 || parsed_gmt->tm_mon > 11)
        return -1;
    if (parsed_gmt->tm_year < 70 || parsed_gmt->tm_year > 120)
        return -1;

    return 0;
}

/*
 * Name: modified_since
 * Description: Decides whether a file's mtime is newer than the
 * If-Modified-Since header of a request.
 *

 * RETURN VALUES:
 *  0: File has not been modified since specified time.
 *  >0: File has been (and value is the converted_time)
 * -1: Error!
 */

int modified_since(time_t * mtime, const char *if_modified_since)
{
    struct tm *file_gmt;
    struct tm parsed_gmt;
    int comp;

    if (date_to_tm(&parsed_gmt, if_modified_since) != 0) {
        return -1;
    }

    file_gmt = gmtime(mtime);

    /* modified by arvin tai 2005.08.09 */
    return 1;
    /* modified end */

    /* Go through from years to seconds -- if they are ever unequal,
       we know which one is newer and can return */
    if ((comp = file_gmt->tm_year - parsed_gmt.tm_year))
        return (comp > 0);
    if ((comp = file_gmt->tm_mon - parsed_gmt.tm_mon))
        return (comp > 0);
    if ((comp = file_gmt->tm_mday - parsed_gmt.tm_mday))
        return (comp > 0);
    if ((comp = file_gmt->tm_hour - parsed_gmt.tm_hour))
        return (comp > 0);
    if ((comp = file_gmt->tm_min - parsed_gmt.tm_min))
        return (comp > 0);
    if ((comp = file_gmt->tm_sec - parsed_gmt.tm_sec))
        return (comp > 0);

    /* this person must really be into the latest/greatest */
    return 0;
}

/*
 * Name: to_upper
 *
 * Description: Turns a string into all upper case (for HTTP_ header forming)
 * AND changes - into _
 */

char *to_upper(char *str)
{
    char *start = str;

    while (*str) {
        if (*str == '-')
            *str = '_';
        else
            *str = toupper(*str);
        str++;
    }

    return start;
}

/*
 * Name: unescape_uri
 *
 * Description: Decodes a uri, changing %xx encodings with the actual
 * character.  The query_string should already be gone.
 *
 * Return values:
 *  1: success
 *  0: illegal string
 */

int unescape_uri(char *uri, char **query_string)
{
    char c, d;
    char *uri_old;

    uri_old = uri;


    while ((c = *uri_old)) {
        if (c == '%') {
            uri_old++;
            if ((c = *uri_old++) && (d = *uri_old++)) {
                *uri = HEX_TO_DECIMAL(c, d);
                if (*uri < 32 || *uri > 126) {
                    /* control chars in URI */
                    *uri = '\0';
                    return 0;
                }
            } else {
                *uri = '\0';
                return 0;
            }
            ++uri;
        } else if (c == '?') {  /* query string */
            if (query_string)
                *query_string = ++uri_old;
            /* stop here */

            *uri = '\0';
            return (1);
        } else if (c == '#') {  /* fragment */
            /* legal part of URL, but we do *not* care.
             * However, we still have to look for the query string */
            if (query_string) {
                ++uri_old;
                while ((c = *uri_old)) {
                    if (c == '?') {
                        *query_string = ++uri_old;
                        break;
                    }
                    ++uri_old;
                }
            }
            break;
        } else {
            *uri++ = c;
            uri_old++;
        }
    }

    *uri = '\0';
    return 1;
}

/* rfc822 (1123) time is exactly 29 characters long
 * "Sun, 06 Nov 1994 08:49:37 GMT"
 */

void rfc822_time_buf(char *buf, time_t s)
{
    struct tm *t;
    char *p;
    unsigned int a;

    if (!s) {
        t = gmtime(&current_time);
    } else
        t = gmtime(&s);

    p = buf + 28;
    /* p points to the last char in the buf */

    p -= 3;
    /* p points to where the ' ' will go */
    memcpy(p--, " GMT", 4);

    a = t->tm_sec;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p-- = ':';
    a = t->tm_min;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p-- = ':';
    a = t->tm_hour;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p-- = ' ';
    a = 1900 + t->tm_year;
    while (a) {
        *p-- = '0' + a % 10;
        a /= 10;
    }
    /* p points to an unused spot to where the space will go */
    p -= 3;
    /* p points to where the first char of the month will go */
    memcpy(p--, month_tab + 4 * (t->tm_mon), 4);
    *p-- = ' ';
    a = t->tm_mday;
    *p-- = '0' + a % 10;
    *p-- = '0' + a / 10;
    *p-- = ' ';
    p -= 3;
    memcpy(p, day_tab + t->tm_wday * 4, 4);
}

char *simple_itoa(unsigned int i)
{
    /* 21 digits plus null terminator, good for 64-bit or smaller ints
     * for bigger ints, use a bigger buffer!
     *
     * 4294967295 is, incidentally, MAX_UINT (on 32bit systems at this time)
     * and is 10 bytes long
     */
    static char local[22];
    char *p = &local[21];
    *p = '\0';
    do {
        *--p = '0' + i % 10;
        i /= 10;
    } while (i != 0);
    return p;
}

/* I don't "do" negative conversions
 * Therefore, -1 indicates error
 */

int boa_atoi(const char *s)
{
    int retval;
    char *reconv;

    if (!isdigit(*s))
        return -1;

    retval = atoi(s);
    if (retval < 0)
        return -1;

    reconv = simple_itoa((unsigned int) retval);
    if (memcmp(s, reconv, strlen(s)) != 0) {
        return -1;
    }
    return retval;
}

int create_temporary_file(short want_unlink, char *storage, unsigned int size)
{
    static char boa_tempfile[MAX_PATH_LENGTH + 1];
    int fd;

    snprintf(boa_tempfile, MAX_PATH_LENGTH, "%s/boa-temp.XXXXXX", tempdir);

    /* open temp file */
    fd = mkstemp(boa_tempfile);
    if (fd == -1) {
        log_error_time();
        perror("mkstemp");
        return 0;
    }

    if (storage != NULL) {
        unsigned int len = strlen(boa_tempfile);

        if (len < size) {
            memcpy(storage, boa_tempfile, len + 1);
        } else {
            close(fd);
            fd = 0;
            log_error_time();
            fprintf(stderr, "not enough memory for memcpy in storage\n");
            want_unlink = 1;
        }
    }

    if (want_unlink) {
        if (unlink(boa_tempfile) == -1) {
            close(fd);
            fd = 0;
            log_error_time();
            fprintf(stderr, "unlink temp file\n");
        }
    }

    return (fd);
}

int real_set_block_fd(int fd)
{
    int flags;

    flags = fcntl(fd, F_GETFL);
    if (flags == -1)
        return -1;

    flags &= ~NOBLOCK;
    flags = fcntl(fd, F_SETFL, flags);
    return flags;
}

int real_set_nonblock_fd(int fd)
{
    int flags;

    flags = fcntl(fd, F_GETFL);
    if (flags == -1)
        return -1;

    flags |= NOBLOCK;
    flags = fcntl(fd, F_SETFL, flags);
    return flags;
}

/* Quoting from rfc1034:

<domain> ::= <subdomain> | " "

<subdomain> ::= <label> | <subdomain> "." <label>

<label> ::= <letter> [ [ <ldh-str> ] <let-dig> ]

<ldh-str> ::= <let-dig-hyp> | <let-dig-hyp> <ldh-str>

<let-dig-hyp> ::= <let-dig> | "-"

<let-dig> ::= <letter> | <digit>

<letter> ::= any one of the 52 alphabetic characters A through Z in
upper case and a through z in lower case

<digit> ::= any one of the ten digits 0 through 9

and

The labels must follow the rules for ARPANET host names.  They must
start with a letter, end with a letter or digit, and have as interior
characters only letters, digits, and hyphen.  There are also some
restrictions on the length.  Labels must be 63 characters or less.

*/

int check_host(const char *r)
{
    /* a hostname can only consist of
     * chars and numbers, and sep. by only
     * one period.
     * It may not end with a period, and must
     * not start with a number.
     *
     * >0: correct
     * -1: error
     *  0: not returned
     *
     */
    const char *c;
    short period_ok = 0;
    short len = 0;

    c = r;
    if (c == NULL) {
        return -1;
    }

    /* must start with a letter or number */
    if (!isalnum(*c))
        return -1;

    if (strlen(c) > 63)
        return -1;

    len = 1;
    while (*(++c) != '\0') {
        /* interior letters may be alphanumeric, '-', or '.' */
        /* '.' may not follow '.' */
        if (isalnum(*c) || *c == '-')
            period_ok = 1;
        else if (*c == '.' && period_ok)
            period_ok = 0;
        else
            return -1;
        ++len;
    }
    /* c points to '\0' */
    --c;
    /* must end with a letter or digit */
    if (!isalnum(*c))
        return -1;
    return len;
}

void strlower(char *s)
{
    while (*s != '\0') {
        *s = tolower(*s);
        ++s;
    }
}

/*
 * Name: base64decode
 *
 * Description: Decodes BASE-64 encoded string
 */
int base64decode(void *dst,char *src,int maxlen)
{
 int bitval,bits;
 int val;
 int len,x,y;

 len = strlen(src);
 bitval=0;
 bits=0;
 y=0;

 for(x=0;x<len;x++)
  {
   if ((src[x]>='A')&&(src[x]<='Z')) val=src[x]-'A'; else
   if ((src[x]>='a')&&(src[x]<='z')) val=src[x]-'a'+26; else
   if ((src[x]>='0')&&(src[x]<='9')) val=src[x]-'0'+52; else
   if (src[x]=='+') val=62; else
   if (src[x]=='-') val=63; else
    val=-1;
   if (val>=0)
    {
     bitval=bitval<<6;
     bitval+=val;
     bits+=6;
     while (bits>=8)
      {
       if (y<maxlen)
        ((char *)dst)[y++]=(bitval>>(bits-8))&0xFF;
       bits-=8;
       bitval &= (1<<bits)-1;
      }
    }
  }
 if (y<maxlen)
   ((char *)dst)[y++]=0;
 return y;
}

static char base64chars[64] = "abcdefghijklmnopqrstuvwxyz"
                              "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";

/*
 * Name: base64encode()
 *
 * Description: Encodes a buffer using BASE64.
 */
void base64encode(unsigned char *from, char *to, int len)
{
  while (len) {
    unsigned long k;
    int c;

    c = (len < 3) ? len : 3;
    k = 0;
    len -= c;
    while (c--)
      k = (k << 8) | *from++;
    *to++ = base64chars[ (k >> 18) & 0x3f ];
    *to++ = base64chars[ (k >> 12) & 0x3f ];
    *to++ = base64chars[ (k >> 6) & 0x3f ];
    *to++ = base64chars[ k & 0x3f ];
  }
  *to++ = 0;
}

#ifndef DISABLE_DEBUG
struct dbg {
    int level;
    char *mesg;
};

static struct dbg debug_level_table[] = {
    {DEBUG_ALIAS, "Alias"},
    {DEBUG_CGI_OUTPUT, "CGI Output"},
    {DEBUG_CGI_INPUT, "CGI Input"},
    {DEBUG_CGI_ENV, "CGI Environment"},
    {DEBUG_HEADER_READ, "Header Read State"},
    {DEBUG_PIPELINE, "Pipeline"},
    {DEBUG_PLUGIN_ERRORS, "Plugin Error"},
    {DEBUG_RANGE, "Range related"},
    {DEBUG_CONFIG, "Configuration"},
    {DEBUG_BUFFER_IO, "Buffer I/O"},
    {DEBUG_BODY_READ, "Body Read State"},
    {DEBUG_MMAP_CACHE, "mmap Cache"},
    {DEBUG_REQUEST, "Generic Request"},
    {DEBUG_HASH, "hash table"}
};


void print_debug_usage(void)
{
    struct dbg *p;

    fprintf(stderr,
            "  To calculate the debug level, logically 'or'\n"
            "  some of the following values together to get a debug level:\n");
    for (p = debug_level_table;
         p <
         debug_level_table +
         (sizeof (debug_level_table) / sizeof (struct dbg)); p++) {
        fprintf(stderr, "\t%d:\t%s\n", p->level, p->mesg);
    }
    fprintf(stderr, "\n");
}

void parse_debug(char *foo)
{
    int i;
    struct dbg *p;

    if (!foo)
        return;

    log_error_time();
    fprintf(stderr, "Before parse_debug, debug_level is: %d\n",
            debug_level);
    if (foo[0] == '-') {
        i = boa_atoi(foo + 1);
        if (i == -1) {
            /* error */
            fprintf(stderr, "Invalid level specified.\n");
            exit(EXIT_FAILURE);
        }
        i = -i;
    } else {
        i = boa_atoi(foo);
        if (i == -1) {
            /* error */
            fprintf(stderr, "Invalid level specified.\n");
            exit(EXIT_FAILURE);
        }
    }
    for (p = debug_level_table;
         p <
         debug_level_table +
         (sizeof (debug_level_table) / sizeof (struct dbg)); p++) {
        if (i > 0) {
            if (i & p->level) {
                log_error_time();
                fprintf(stderr, "Enabling %s debug level.\n",
                        p->mesg);
                debug_level |= p->level;
            }
        } else {
            if (-i & p->level) {
                log_error_time();
                fprintf(stderr, "Disabling %s debug level.\n",
                        p->mesg);
                debug_level &= ~(p->level);
            }
        }
    }
    log_error_time();
    fprintf(stderr, "After parse_debug, debug_level is: %d\n",
            debug_level);
}
#endif

/* 2004.12.21 add by arvin tai */
int save_config_file()
{
    FILE *wlanconf;
    char *wordlist=NULL;
    
    wlanconf = fopen("/etc/Wireless/RT61AP/RT61AP.dat", "w+");
    
    fprintf(wlanconf, "#The word of \"Default\" must not be removed\n");
    fprintf(wlanconf, "Default\n");
    //CountryRegion
    wordlist = nvram_get("CountryRegion");
    fprintf(wlanconf, "CountryRegion=%d\n", atoi(wordlist));
    //CountryRegion for A band
    wordlist = nvram_get("CountryRegionABand");
    fprintf(wlanconf, "CountryRegionABand=%d\n", atoi(wordlist));
   	//CountryCode
   	if (strcmp(nvram_get("CountryCode"), "") == 0)
   	   fprintf(wlanconf, "CountryCode=\n");
   	else
   	   fprintf(wlanconf, "CountryCode=%s\n", nvram_get("CountryCode"));
    //SSID Num. [MSSID Only]
    wordlist = nvram_get("SSIDNum");
    fprintf(wlanconf, "BssidNum=%d\n", atoi(wordlist));
   	//SSID
   	fprintf(wlanconf, "SSID=%s\n", nvram_get("SSID"));
   	//Network Mode
   	wordlist = nvram_get("WirelessMode");
  	fprintf(wlanconf, "WirelessMode=%d\n", atoi(wordlist));
    //TxRate(FixedRate)
    fprintf(wlanconf, "TxRate=%s\n", nvram_get("TxRate"));
    //Channel
    wordlist = nvram_get("Channel");
    fprintf(wlanconf, "Channel=%d\n", atoi(wordlist));
    //BasicRate
    wordlist = nvram_get("BasicRate");
    fprintf(wlanconf, "BasicRate=%d\n", atoi(wordlist));
    //BeaconPeriod
    wordlist = nvram_get("BeaconPeriod");
    fprintf(wlanconf, "BeaconPeriod=%d\n", atoi(wordlist));
    //DTIM Period
    wordlist = nvram_get("DtimPeriod");
    fprintf(wlanconf, "DtimPeriod=%d\n", atoi(wordlist));
    //TxPower
    wordlist = nvram_get("TxPower");
    fprintf(wlanconf, "TxPower=%d\n", atoi(wordlist));
    //BGProtection
    wordlist = nvram_get("BGProtection");
    fprintf(wlanconf, "BGProtection=%d\n", atoi(wordlist));
    //TxAntenna
    fprintf(wlanconf, "TxAntenna=\n");
    //wordlist = nvram_get("TxAntenna");
    //fprintf(wlanconf, "TxAntenna=%d\n", atoi(wordlist));
    //RxAntenna
    fprintf(wlanconf, "RxAntenna=\n");
    //wordlist = nvram_get("RxAntenna");
    //fprintf(wlanconf, "RxAntenna=%d\n", atoi(wordlist));
    //TxPreamble
    wordlist = nvram_get("TxPreamble");
    fprintf(wlanconf, "TxPreamble=%d\n", atoi(wordlist));
    //RTSThreshold  Default=2347
    wordlist = nvram_get("RTSThreshold");
    fprintf(wlanconf, "RTSThreshold=%d\n", atoi(wordlist));
    //FragThreshold  Default=2346
    wordlist = nvram_get("FragThreshold");
    fprintf(wlanconf, "FragThreshold=%d\n", atoi(wordlist));
    //TxBurst
    wordlist = nvram_get("TxBurst");
    fprintf(wlanconf, "TxBurst=%d\n", atoi(wordlist));
    //PktAggregate
    wordlist = nvram_get("PktAggregate");
    fprintf(wlanconf, "PktAggregate=%d\n", atoi(wordlist));
    //TurboRate
    //wordlist = nvram_get("TurboRate");
    fprintf(wlanconf, "TurboRate=0\n");
    //WmmCapable
    wordlist = nvram_get("WmmCapable");
    fprintf(wlanconf, "WmmCapable=%d\n", atoi(wordlist));
    //APAifsn
    fprintf(wlanconf, "APAifsn=7;3;1;1\n");
    //APCwmin
    fprintf(wlanconf, "APCwmin=15;15;7;3\n");
    //APCwmax
    fprintf(wlanconf, "APCwmax=1023;63;15;7\n");
    //APTxop
    fprintf(wlanconf, "APTxop=0;0;94;47\n");
    //APACM
    fprintf(wlanconf, "APACM=0;0;0;0\n");
    //BSSAifsn
    fprintf(wlanconf, "BSSAifsn=7;3;2;2\n");
    //BSSCwmin
    fprintf(wlanconf, "BSSCwmin=15;15;7;3\n");
    //BSSCwmax
    fprintf(wlanconf, "BSSCwmax=1023;1023;15;7\n");
    //BSSTxop
    fprintf(wlanconf, "BSSTxop=0;0;94;47\n");
    //BSSACM
    fprintf(wlanconf, "BSSACM=0;0;0;0\n");
    //AckPolicy
    fprintf(wlanconf, "AckPolicy=0;0;0;0\n");
    //NoForwarding pre SSID
    fprintf(wlanconf, "NoForwarding=%s\n", nvram_get("NoForwarding"));
    //NoForwardingBTNBSSID
    wordlist = nvram_get("NoForwardingBTNBSSID");
    fprintf(wlanconf, "NoForwardingBTNBSSID=%d\n", atoi(wordlist));
    //HideSSID
    fprintf(wlanconf, "HideSSID=%s\n", nvram_get("HideSSID"));
    //ShortSlot
    wordlist = nvram_get("ShortSlot");
    fprintf(wlanconf, "ShortSlot=%d\n", atoi(wordlist));
    //AutoChannelSelect
    wordlist = nvram_get("AutoChannelSelect");
    fprintf(wlanconf, "AutoChannelSelect=%d\n", atoi(wordlist));
    //IEEE8021X
    fprintf(wlanconf, "IEEE8021X=%s\n", nvram_get("IEEE8021X"));
    //IEEE80211H
    wordlist = nvram_get("IEEE80211H");
    fprintf(wlanconf, "IEEE80211H=%d\n", atoi(wordlist));
    //CSPeriod
    fprintf(wlanconf, "CSPeriod=6\n");
    //PreAuth
    fprintf(wlanconf, "PreAuth=%s\n", nvram_get("PreAuth"));
    //AuthMode
    fprintf(wlanconf, "AuthMode=%s\n", nvram_get("AuthMode"));
    //EncrypType
    fprintf(wlanconf, "EncrypType=%s\n", nvram_get("EncrypType"));
    //RekeyInterval
    wordlist = nvram_get("RekeyInterval");
    fprintf(wlanconf, "RekeyInterval=%d\n", atoi(wordlist));
    //RekeyMethod
    fprintf(wlanconf, "RekeyMethod=%s\n", nvram_get("RekeyMethod"));
    //PMKCachePeriod
    wordlist = nvram_get("PMKCachePeriod");
    fprintf(wlanconf, "PMKCachePeriod=%d\n", atoi(wordlist));
    //WPAPSK
    fprintf(wlanconf, "WPAPSK=%s\n", nvram_get("WPAPSK"));
    //DefaultKeyID
    fprintf(wlanconf, "DefaultKeyID=%s\n", nvram_get("DefaultKeyID"));
    //Key1Type(0 -> Hex, 1->Ascii)
    fprintf(wlanconf, "Key1Type=%s\n", nvram_get("Key1Type"));
    //Key1Str
    fprintf(wlanconf, "Key1Str=%s\n", nvram_get("Key1Str"));
    //Key2Type
   	fprintf(wlanconf, "Key2Type=%s\n", nvram_get("Key2Type"));
    //Key2Str
    fprintf(wlanconf, "Key2Str=%s\n", nvram_get("Key2Str"));
    //Key3Type
    fprintf(wlanconf, "Key3Type=%s\n", nvram_get("Key3Type"));
    //Key3Str
    fprintf(wlanconf, "Key3Str=%s\n", nvram_get("Key3Str"));
    //Key4Type
    fprintf(wlanconf, "Key4Type=%s\n", nvram_get("Key4Type"));
    //Key4Str
    fprintf(wlanconf, "Key4Str=%s\n", nvram_get("Key4Str"));
    //HSCounter
    wordlist = nvram_get("HSCounter");
    fprintf(wlanconf, "HSCounter=%d\n", atoi(wordlist));
    //AccessPolicy0
    wordlist = nvram_get("AccessPolicy0");
    fprintf(wlanconf, "AccessPolicy0=%d\n", atoi(wordlist));
    //AccessControlLis0t
    fprintf(wlanconf, "AccessControlList0=%s\n", nvram_get("AccessControlList0"));
    //AccessPolicy1
    wordlist = nvram_get("AccessPolicy1");
    fprintf(wlanconf, "AccessPolicy1=%d\n", atoi(wordlist));
    //AccessControlList1
    fprintf(wlanconf, "AccessControlList1=%s\n", nvram_get("AccessControlList1"));
    //AccessPolicy2
    wordlist = nvram_get("AccessPolicy2");
    fprintf(wlanconf, "AccessPolicy2=%d\n", atoi(wordlist));
    //AccessControlList2
    fprintf(wlanconf, "AccessControlList2=%s\n", nvram_get("AccessControlList2"));
    //AccessPolicy3
    wordlist = nvram_get("AccessPolicy3");
    fprintf(wlanconf, "AccessPolicy3=%d\n", atoi(wordlist));
    //AccessControlList3
    fprintf(wlanconf, "AccessControlList3=%s\n", nvram_get("AccessControlList3"));
    //WdsEnable
    wordlist = nvram_get("WdsEnable");
    fprintf(wlanconf, "WdsEnable=%d\n", atoi(wordlist));
    //WdsList
    fprintf(wlanconf, "WdsList=%s\n", nvram_get("WdsList"));
    //WdsKey
    fprintf(wlanconf, "WdsKey=\n");
    //RADIUS_Server
    fprintf(wlanconf, "RADIUS_Server=%s\n", nvram_get("RADIUS_Server"));
    //RADIUS_Port
    wordlist = nvram_get("RADIUS_Port");
    fprintf(wlanconf, "RADIUS_Port=%d\n", atoi(wordlist));
    //RADIUS_Key
    fprintf(wlanconf, "RADIUS_Key=%s\n", nvram_get("RADIUS_Key"));
    //own_ip_addr
    fprintf(wlanconf, "own_ip_addr=%s\n", nvram_get("own_ip_addr"));
    //session_timeout_interval
    wordlist = nvram_get("session_timeout_interval");
    fprintf(wlanconf, "session_timeout_interval=%d\n", atoi(wordlist));
    
    fclose(wlanconf);
    
    return 1;
}

void
getifInfo(char *ifname, char *if_hw, char *if_addr, char *if_net)
{
   struct ifconf ifc;
   char buf[BUFSIZ];
   struct ifreq *pifr;
   int n;
   
   if((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      perror("socket");
	  exit(EXIT_FAILURE);
   }
   
   ifc.ifc_len = sizeof(buf);
   ifc.ifc_buf = buf;

   if(ioctl(skfd, SIOCGIFCONF, (char *) &ifc) < 0) {
      perror("SIOCGIFCONF");
      return;
   }
   
   pifr = ifc.ifc_req;
   for(n = ifc.ifc_len/sizeof(struct ifreq); --n >= 0; pifr++) {
      if(((pifr->ifr_flags & IFF_UP) == 0) && (strcmp(pifr->ifr_name, ifname) == 0)) {
	     do_one(skfd, pifr->ifr_name);
	     sprintf(if_hw, "%s", g_hw);
	     sprintf(if_addr, "%s", g_addr);
	     sprintf(if_net, "%s", g_net);
	  }
   }
   
   (void) close(skfd);
}

void
do_one(int sk, char* ifname)
{
  struct ifreq ifr;
  char *ptr;

  strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
  if(ioctl(sk, SIOCGIFFLAGS, &ifr) < 0) {
    perror("SIOCGIFFLAGS");
    return;
  }
  g_flags = ifr.ifr_flags;

  /* hardware address */
  strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
  if(ioctl(sk, SIOCGIFHWADDR, &ifr) < 0) {
	perror("SIOCGIFHWADDR");
	return;
  }

  ptr = (char *)&ifr.ifr_addr.sa_data;
  sprintf(g_hw, "%02X:%02X:%02X:%02X:%02X:%02X",
		  (ptr[0] & 0377), (ptr[1] & 0377), (ptr[2] & 0377),
		  (ptr[3] & 0377), (ptr[4] & 0377), (ptr[5] & 0377));

  if(g_flags & IFF_UP) {
    /* interface address */
    strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
    if(ioctl(sk, SIOCGIFADDR, &ifr) < 0) {
      perror("SIOCGIFDADDR");
      return;
    }
    strcpy(g_addr, inet_ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));
		
    /* netmask */
    strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
    if(ioctl(sk, SIOCGIFNETMASK, &ifr) < 0) {
      perror("SIOCGIFNETMASK");
      return;
    }
	strcpy(g_net, inet_ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));
		
    /* broadcast address */
    strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
    if(ioctl(sk, SIOCGIFBRDADDR, &ifr) < 0) {
      perror("SIOCGIFBRDADDR");
      return;
    }
	strcpy(g_brd, inet_ntoa(((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr));
  }
}

void
getWlanMacAddress(char *ifname, char *if_hw)
{
  struct ifreq ifr;
  char *ptr;
  int skfd;

   if((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      perror("socket");
      exit(EXIT_FAILURE);
   }

  /* hardware address */
  strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
  if(ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) {
	perror("SIOCGIFHWADDR");
	return;
  }

  ptr = (char *)&ifr.ifr_addr.sa_data;
  sprintf(if_hw, "%02X:%02X:%02X:%02X:%02X:%02X",
		  (ptr[0] & 0377), (ptr[1] & 0377), (ptr[2] & 0377),
		  (ptr[3] & 0377), (ptr[4] & 0377), (ptr[5] & 0377));
		  
  (void) close(skfd);
}

void
setBridgeMacAddress(char *ifname)
{
  struct ifreq ifr;

   if((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      perror("socket");
      exit(EXIT_FAILURE);
   }

  /* hardware address */
  strncpy(ifr.ifr_name, ifname, IF_NAMESIZE);
  if(ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) {
	perror("SIOCGIFHWADDR");
	return;
  }

  strncpy(ifr.ifr_name, "br0", IFNAMSIZ);
  ifr.ifr_hwaddr.sa_family = ARPHRD_ETHER;
  ioctl(skfd, SIOCSIFHWADDR, &ifr);

  (void) close(skfd);
}

int start_dhcpd(){
	
    FILE *fp=NULL;
    char *value=NULL;
    char wlanbuf[128] = {0};
    int  lease = 0;
    int  ret;

	//Open udhcpd lease file
	if (!(fp = fopen("/etc/udhcpd.leases", "a"))) {
		perror("/etc/udhcpd.leases");
		return errno;
	}
	fclose(fp);

	//Write udhcpd config file
	if (!(fp = fopen("/etc/udhcpd.conf", "w"))) {
		perror("/etc/udhcpd.conf");
		return errno;
	}
    fprintf(fp, "start %s\n", nvram_get("dhcpStart"));
    fprintf(fp, "end %s\n", nvram_get("dhcpEnd"));
    fprintf(fp, "interface br0\n");
    fprintf(fp, "remaining yes\n");
    fprintf(fp, "lease_file /etc/udhcpd.leases\n");
    if (strcmp(nvram_get("dns_relay_enabled"), "1") == 0)
       fprintf(fp, "opt dns %s\n", nvram_get("lan_ipaddr"));
    else
       fprintf(fp, "opt dns %s %s\n", nvram_get("wan_primary_dns"), nvram_get("wan_secondary_dns"));
    fprintf(fp, "option subnet %s\n", nvram_get("lan_netmask"));
    fprintf(fp, "opt router %s\n", nvram_get("lan_ipaddr"));
    value = nvram_get("dhcpLease");
    lease = atoi(value)*60;

	if(lease <= 0)
	  fprintf(fp, "option lease %d\n", 0);
	else
	  fprintf(fp, "option lease %d\n", lease);

    fclose(fp);

    memset(wlanbuf, 0, 128);
    sprintf(wlanbuf, "/usr/sbin/udhcpd");
    ret = system(wlanbuf);
#ifdef USE_DEBUG
    if (ret)
       fprintf(stderr, "boa:  (%s) fail !!!!!!!!!!!!!!!!!!!!\n", wlanbuf);
    else
       fprintf(stderr, "boa: wlanbuf (%s) !!!!!!!!!!!!!!!!!!!!!!!!\n", wlanbuf);
#endif

	return 1;
}

int stop_dhcpd(){

    char   wlanbuf[128] = {0};
    int    ret;

    memset(wlanbuf, 0, 128);
    sprintf(wlanbuf, "/usr/bin/killall udhcpd");
    ret = system(wlanbuf);
#ifdef USE_DEBUG
    if (ret)
       fprintf(stderr, "boa:  (%s) fail !!!!!!!!!!!!!!!!!!!!\n", wlanbuf);
    else
       fprintf(stderr, "boa: wlanbuf (%s) !!!!!!!!!!!!!!!!!!!!!!!!\n", wlanbuf);
#endif

	printf("stop_dhcpd done\n");
	return 1;
}
