/*
 * (C) Copyright 2000-2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <unistd.h>
#include "ram_env.h"
#include <fcntl.h>

#if !defined(RAM_FILE)		

#ifdef KerVer_4
	#include <linux/mtd/mtd.h>
#else // Version_6	
	#include <mtd/mtd-user.h>
#endif

#endif
typedef unsigned char uchar;


typedef struct envdev_s {
	uchar devname[16];		/* Device name */
	ulong devoff;			/* Device offset */
	ulong env_size;			/* environment size */
	ulong erase_size;		/* device erase size */
} envdev_t;

static envdev_t envdevices[2];
static int curdev;
enum {false=0,true};

#define DEVNAME(i)    envdevices[(i)].devname
#define DEVOFFSET(i)  envdevices[(i)].devoff
#define ENVSIZE(i)    envdevices[(i)].env_size
#define DEVESIZE(i)   envdevices[(i)].erase_size
#define MAX_CACHE_ENV_ENTRY		4096

#define CFG_ENV_SIZE ENVSIZE(curdev)

#define ENV_SIZE      getenvsize()

typedef struct environment_s {
	ulong crc;			/* CRC32 over data bytes    */
	uchar flags;			/* active or obsolete */
	uchar *data;
} env_t;

typedef struct cache_environment_s {
	uchar *name;
	uchar *value;
} cache_t;

static env_t environment;

static cache_t cache_env[MAX_CACHE_ENV_ENTRY];

static int HaveRedundEnv = 0;

static uchar active_flag = 1;
static int buffer_opt=0;   	
static uchar obsolete_flag = 0;


#define XMK_STR(x)	#x
#define MK_STR(x)	XMK_STR(x)

static uchar default_environment[ENV1_SIZE] = {
#ifdef RT61AP
	"SystemName=RalinkAP\0"
	"Login=admin\0"
	"Password=admin\0"
	"getTimeMethod=PC\0"
	"ntp_server=220.130.158.51\0"
	"ntp_intervel=24\0"
	"timeZone=46\0"
	"user_time_yr=\0"
	"user_time_mo=\0"
	"user_time_dd=\0"
	"user_time_hr=\0"
	"user_time_mn=\0"
	"wanConnectionMode=STATIC\0"
	"wan_ipaddr=192.168.5.201\0"
	"wan_netmask=255.255.255.0\0"
	"wan_gateway=192.168.5.254\0"
	"wan_primary_dns=192.168.1.5\0"
	"wan_secondary_dns=168.95.1.1\0"
	"wan_wins=\0"
	"clone_macaddr=\0"
	"lan_ipaddr=192.168.100.1\0"
	"lan_netmask=255.255.255.0\0"
	"dhcpEnabled=1\0"
	"dhcpStart=192.168.100.2\0"
	"dhcpEnd=192.168.100.254\0"
	"dhcpLease=1440\0"
	"dns_relay_enabled=0\0"
	"option_dns=168.95.192.1\0"
	"RadioEnable=1\0"
	"CountryRegion=5\0"
	"CountryRegionABand=7\0"
	"CountryCode=\0"
	"SSIDNum=1\0"
	"SSID=ralink-demo\0"
	"SSIDPos=1;0;0;0\0"
	"WirelessMode=0\0"
	"TxRate=0\0"
	"Channel=7\0"
	"BasicRate=15\0"
	"BeaconPeriod=100\0"
	"DtimPeriod=1\0"
	"TxPower=100\0"
	"DisableOLBC=0\0"
	"BGProtection=0\0"
	"TxPreamble=0\0"
	"RTSThreshold=2347\0"
	"FragThreshold=2346\0"
	"TxBurst=0\0"
	"PktAggregate=0\0"
	"TurboRate=0\0"
	"WmmCapable=0\0"
	"APSDCapable=0\0"
	"DLSCapable=0\0"
	"APAifsn=3;7;1;1\0"
	"APCwmin=4;4;3;2\0"
	"APCwmax=6;10;4;3\0"
	"APTxop=0;0;94;47\0"
	"APACM=0;0;0;0\0"
	"AckPolicy=0;0;0;0\0"
	"BSSAifsn=3;7;2;2\0"
	"BSSCwmin=4;4;3;2\0"
	"BSSCwmax=10;10;4;3\0"
	"BSSTxop=0;0;94;47\0"
	"BSSACM=0;0;0;0\0"
	"NoForwarding=0\0"
	"NoForwardingBTNBSSID=0\0"
	"HideSSID=0\0"
	"ShortSlot=1\0"
	"AutoChannelSelect=0\0"
	"SecurityMode=0\0"
	"UAMIPPool=192.168.182.0/24\0"
	"UAMDhcpIF=\0"
	"UAMDns1=\0"
	"UAMDns2=\0"
	"UAMLeaseTime=600\0"
	"UAMRadiusNASID=nas01\0"
	"UAMRadius1=\0"
	"UAMRadius2=\0"
	"UAMRadiusSecret=\0"
	"UAMUrl=https://192.168.1.25/cgi-bin/hotspotlogin.cgi\0"
	"VLANEnable=0\0"
	"VLANName=\0"
	"VLANID=0\0"
	"VLANPriority=0\0"
	"IEEE8021X=0\0"
	"IEEE80211H=0\0"
	"PreAuth=0\0"
	"AuthMode=OPEN\0"
	"EncrypType=NONE\0"
	"RekeyInterval=3600\0"
	"RekeyMethod=DISABLE\0"
	"PMKCachePeriod=10\0"
	"WPAPSK=\0"
	"DefaultKeyID=1\0"
	"Key1Type=0\0"
	"Key1Str=\0"
	"Key2Type=0\0"
	"Key2Str=\0"
	"Key3Type=0\0"
	"Key3Str=\0"
	"Key4Type=0\0"
	"Key4Str=\0"
	"HSCounter=0\0"
	"AccessPolicy0=0\0"
	"AccessControlList0=\0"
	"AccessPolicy1=0\0"
	"AccessControlList1=\0"
	"AccessPolicy2=0\0"
	"AccessControlList2=\0"
	"AccessPolicy3=0\0"
	"AccessControlList3=\0"
	"WdsEnable=0\0"
	"WdsEncrypType=NONE\0"
	"WdsList=\0"
	"WdsKey=\0"
	"RADIUS_Server=\0"
	"RADIUS_Port=1812\0"
	"RADIUS_Key=\0"
	"RADIUS_Acct_Server=\0"
	"RADIUS_Acct_Port=1813\0"
	"RADIUS_Acct_Key=\0"
	"session_timeout_interval=0\0"
	"idle_timeout_interval=0\0"
	"own_ip_addr=\0"
	"Ethifname=eth1\0"
#endif
	"\0"			/* Termimate env_t data with 2 NULs */
};

static int flash_io (int mode);
static uchar *envmatch (uchar * s1, uchar * s2);
static int env_init (void);
static int buff_env_init ();
static int parse_config (void);
static int getlock(int fd, int type);
static unsigned char *cache_in (unsigned char *name,unsigned char *val);
#if defined(CONFIG_FILE)
static int get_config (char *);
#endif
static inline ulong getenvsize (void)
{
	ulong rc = CFG_ENV_SIZE - sizeof (long);

	if (HaveRedundEnv)
		rc -= sizeof (char);
	return rc;
}

void ram_init(void)
{
    environment.data=0;
    memset(cache_env,sizeof(cache_env),0);
}

void ram_close(void)
{
    int i;

    for(i=0;i< MAX_CACHE_ENV_ENTRY;i++)
    {	    
    	if(cache_env[i].name)
	    free(cache_env[i].name);
	if(cache_env[i].value)
	    free(cache_env[i].value);
    }	
}

char *ram_get(char *name)
{  
    char * result;
    result = (char *)ram_getenv((unsigned char *)name);
    if(environment.data != default_environment)
	free(environment.data);    
    return result;
}

int ram_set (char *name, char *value)
{
    int result;	
    char *argv[4] = { "setenv", name, value, NULL };
    
    result= ram_setenv (3, argv);
    if(environment.data != default_environment )
	free(environment.data);    
    return result;
}

int ram_bufset (char *name, char *value)
{
    char *argv[4] = { "setenv", name, value, NULL };
    
	return ram_buff_setenv (3, argv);
}

int ram_commit()
{
    int result;
    
    result= do_ram_commit();
    if(environment.data != default_environment )
	free(environment.data);    
    return result;
    
}

int ram_match(char *name,char *value)
{
    char *tmp;
    int ret;

    tmp = ram_get(name);
    if(tmp)
    {
         ret = (!strcmp(tmp,value));
         return ret;
    }
    else
         return false;
}
/*
 * Search the environment for a variable.
 * Return the value, if found, or NULL, if not found.
 */
unsigned char *ram_getenv (unsigned char *name)
{
	uchar *env, *nxt;

	if (env_init ())
		return (NULL);

	for (env = environment.data; *env; env = nxt + 1) {
		uchar *val;

		for (nxt = env; *nxt; ++nxt) {
			if (nxt >= &environment.data[ENV_SIZE]) {
				fprintf (stderr, "## Error: "
					"environment not terminated\n");
				return (NULL);
			}
		}
		val = envmatch (name, env);
		if (!val)
			continue;
		return cache_in(name,val);
	}
	return (NULL);
}

static unsigned char *cache_in (unsigned char *name,unsigned char *val)
{
    int i=0;
    static int next_cache_entry=0;
    
    unsigned char *cache_entry_name;
    unsigned char *cache_entry_value;
    
    for(i=0;i<MAX_CACHE_ENV_ENTRY;i++)
    {
    	if (cache_env[i].name ==0){
		break;
	}	
	if (strcmp((char *)cache_env[i].name, (char *)name)==0)//found 
	{
		if(cache_env[i].value)
		    free(cache_env[i].value);
	        if (( cache_entry_value= (unsigned char *)calloc (1, strlen((char *)val)+1)) == NULL) {
		    printf("can not allocate memory for cache value \n");
		    return NULL;

		}    
		cache_env[i].value=cache_entry_value;
		strcpy((char *)cache_env[i].value, (char *)val);
		return cache_entry_value;
	}
    }
    if (next_cache_entry <MAX_CACHE_ENV_ENTRY)// not found
    {
	if(cache_env[i].value)
	    free(cache_env[i].value);
        if (( cache_entry_value= calloc (1, strlen((char *)val)+1)) == NULL) {
	    printf("can not allocate memory for cache value\n");
	    return NULL;

	}    
	if(cache_env[i].name)
	    free(cache_env[i].name);
        if (( cache_entry_name= calloc (1, strlen((char *)name)+1)) == NULL) {
	    printf("can not allocate memory for cache name tag\n");
	    return NULL;
	}
	cache_env[next_cache_entry].name=cache_entry_name;
	cache_env[next_cache_entry].value=cache_entry_value;
	strcpy((char *)cache_env[next_cache_entry].name, (char *)name);     
	strcpy((char *)cache_env[next_cache_entry].value, (char *)val);     
	next_cache_entry++;
	return cache_entry_value;
    }	
    else
	printf("ERROR: nvram cache full\n");    
    return NULL;
}
/*
 * Print the current definition of one, or more, or all
 * environment variables
 */
void ram_printenv (int argc, char *argv[])
{
	uchar *env, *nxt;
	int i, n_flag;

	if (env_init ())
		return;

	if (argc == 1) {		/* Print all env variables  */
		for (env = environment.data; *env; env = nxt + 1) {
			for (nxt = env; *nxt; ++nxt) {
				if (nxt >= &environment.data[ENV_SIZE]) {
					fprintf (stderr, "## Error: "
						"environment not terminated\n");
					return;
				}
			}

			printf ("%s\n", env);
		}
		return;
	}

	if (strcmp (argv[1], "-n") == 0) {
		n_flag = 1;
		++argv;
		--argc;
		if (argc != 2) {
			fprintf (stderr, "## Error: "
				"`-n' option requires exactly one argument\n");
			return;
		}
	} else {
		n_flag = 0;
	}

	for (i = 1; i < argc; ++i) {	/* print single env variables   */
		uchar *name = (uchar *)argv[i];
		uchar *val = NULL;

		for (env = environment.data; *env; env = nxt + 1) {

			for (nxt = env; *nxt; ++nxt) {
				if (nxt >= &environment.data[ENV_SIZE]) {
					fprintf (stderr, "## Error: "
						"environment not terminated\n");
					return;
				}
			}
			val = envmatch (name, env);
			if (val) {
				if (!n_flag) {
					fputs ((char *)name, stdout);
					putc ('=', stdout);
				}
				puts ((char *)val);
				break;
			}
		}
		if (!val)
			fprintf (stderr, "## Error: \"%s\" not defined\n", name);
	}
}

int buff_env_init( )
{
    // only read once	
    // after commit , buffer_opt will be clean to 0
    if(buffer_opt==0)
    {	    
    	env_init();
	buffer_opt=1;
    }	
    return 0;
} 

int do_ram_commit()
{
	if(buffer_opt==0)
	{
	    printf("nothing to be commit\n");
	    return 0;
	}
	
	/* Update CRC */
	environment.crc = crc32 (0, environment.data, ENV_SIZE);

	/* write environment back to flash */
	if (flash_io (O_RDWR)) {
		fprintf (stderr, "Error: can't write ram_env to flash\n");
		return (-1);
	}
	buffer_opt=0;
	return (0);

}

int ram_buff_setenv (int argc, char *argv[])
{
	int i, len;
	uchar *env, *nxt;
	uchar *oldval = NULL;
	uchar *name;
	
	if (argc < 2) {
		return (EINVAL);
	}
	
	if (buffer_opt==0 && buff_env_init ())
		return (errno);

	name = (uchar *)argv[1];

	/*
	 * search if variable with this name already exists
	 */
	for (nxt = env = environment.data; *env; env = nxt + 1) {
		for (nxt = env; *nxt; ++nxt) {
			if (nxt >= &environment.data[ENV_SIZE]) {
				fprintf (stderr, "## Error: "
					"environment not terminated\n");
				return (EINVAL);
			}
		}
		if ((oldval = envmatch (name, env)) != NULL)
			break;
	}

	/*
	 * Delete any existing definition
	 */
	if (oldval) {
		/*
		 * Ethernet Address and serial# can be set only once
		 */
		if ((strcmp ((char *)name, "ethaddr") == 0) ||
			(strcmp ((char *)name, "serial#") == 0)) {
			fprintf (stderr, "Can't overwrite \"%s\"\n", name);
			return (EROFS);
		}

		if (*++nxt == '\0') {
			*env = '\0';
		} else {
			for (;;) {
				*env = *nxt++;
				if ((*env == '\0') && (*nxt == '\0'))
					break;
				++env;
			}
		}
		*++env = '\0';
	}

	/*
	 * Append new definition at the end
	 */
	for (env = environment.data; *env || *(env + 1); ++env);
	if (env > environment.data)
		++env;
	/*
	 * Overflow when:
	 * "name" + "=" + "val" +"\0\0"  > CFG_ENV_SIZE - (env-environment)
	 */
	len = strlen ((char *)name) + 2;
	/* add '=' for first arg, ' ' for all others */
	for (i = 2; i < argc; ++i) {
		len += strlen (argv[i]) + 1;
	}
	if (len > (&environment.data[ENV_SIZE] - env)) {
		fprintf (stderr,
			"Error: environment overflow, \"%s\" deleted\n",
			name);
		return (-1);
	}
	while ((*env = *name++) != '\0')
		env++;
	for (i = 2; i < argc; ++i) {
		uchar *val = (uchar *)argv[i];

		*env = (i == 2) ? '=' : ' ';
		while ((*++env = *val++) != '\0');
	}

	/* end is marked with double '\0' */
	*++env = '\0';
	return (0);
}

/*
 * Deletes or sets environment variables. Returns errno style error codes:
 * 0	  - OK
 * EINVAL - need at least 1 argument
 * EROFS  - certain variables ("ethaddr", "serial#") cannot be
 *	    modified or deleted
 *
 */
int ram_setenv (int argc, char *argv[])
{
	int i, len;
	uchar *env, *nxt;
	uchar *oldval = NULL;
	uchar *name;

	if (argc < 2) {
		return (EINVAL);
	}

	if (env_init ())
		return (errno);

	name = (uchar *)argv[1];

	/*
	 * search if variable with this name already exists
	 */
	for (nxt = env = environment.data; *env; env = nxt + 1) {
		for (nxt = env; *nxt; ++nxt) {
			if (nxt >= &environment.data[ENV_SIZE]) {
				fprintf (stderr, "## Error: "
					"environment not terminated\n");
				return (EINVAL);
			}
		}
		if ((oldval = envmatch (name, env)) != NULL)
			break;
	}

	/*
	 * Delete any existing definition
	 */
	if (oldval) {
		/*
		 * Ethernet Address and serial# can be set only once
		 */
		if ((strcmp ((char *)name, "ethaddr") == 0) ||
			(strcmp ((char *)name, "serial#") == 0)) {
			fprintf (stderr, "Can't overwrite \"%s\"\n", name);
			return (EROFS);
		}

		if (*++nxt == '\0') {
			*env = '\0';
		} else {
			for (;;) {
				*env = *nxt++;
				if ((*env == '\0') && (*nxt == '\0'))
					break;
				++env;
			}
		}
		*++env = '\0';
	}

	/* Delete only ? */
	if (argc < 3)
		goto WRITE_FLASH;

	/*
	 * Append new definition at the end
	 */
	for (env = environment.data; *env || *(env + 1); ++env);
	if (env > environment.data)
		++env;
	/*
	 * Overflow when:
	 * "name" + "=" + "val" +"\0\0"  > CFG_ENV_SIZE - (env-environment)
	 */
	len = strlen ((char *)name) + 2;
	/* add '=' for first arg, ' ' for all others */
	for (i = 2; i < argc; ++i) {
		len += strlen (argv[i]) + 1;
	}
	if (len > (&environment.data[ENV_SIZE] - env)) {
		fprintf (stderr,
			"Error: environment overflow, \"%s\" deleted\n",
			name);
		return (-1);
	}
	while ((*env = *name++) != '\0')
		env++;
	for (i = 2; i < argc; ++i) {
		uchar *val = (uchar *)argv[i];

		*env = (i == 2) ? '=' : ' ';
		while ((*++env = *val++) != '\0');
	}

	/* end is marked with double '\0' */
	*++env = '\0';

  WRITE_FLASH:

	/* Update CRC */
	environment.crc = crc32 (0, environment.data, ENV_SIZE);

	/* write environment back to flash */
	if (flash_io (O_RDWR)) {
		fprintf (stderr, "Error: can't write ram_env to flash\n");
		return (-1);
	}

	return (0);
}

static int getlock(int fd, int type)
{
	struct flock lockinfo;
	lockinfo.l_whence = SEEK_SET;
	lockinfo.l_start = 0;
	lockinfo.l_len =0;

	while (1)
	{
	    lockinfo.l_type=type;
	    if (!fcntl(fd,F_SETLK,&lockinfo)) return 0;
#ifdef DEBUG	    
	    if(type==F_RDLCK) 
	        printf("can not do READLOCK\n");
	    if(type==F_WRLCK)
		    printf("can not do WRITE LOCK \n");
#endif	    
	    fcntl(fd,F_GETLK,&lockinfo);
#ifdef DEBUG	    
	    if(lockinfo.l_type != F_UNLCK)
	    	printf("process %d conflict with process %d \n",getpid(),lockinfo.l_pid);
#endif	    
	    sleep(1);
		    
	}    
			    
}
static int flash_io (int mode)
{
	int fd, fdr, rc, otherdev, len, resid;
#if !defined(RAM_FILE)
	erase_info_t erase;
#endif
	char *data = NULL;

	if ((fd = open ((char *)DEVNAME (curdev), mode)) < 0) {
		fprintf (stderr,
			"Can't open %s: %s\n",
			DEVNAME (curdev), strerror (errno));
		return (-1);
	}

	len = sizeof (environment.crc);
	if (HaveRedundEnv) {
		len += sizeof (environment.flags);
	}

	if (mode == O_RDWR) {
#ifdef DEBUG		
		printf("getting write lock\n");
#endif		
		getlock(fd,F_WRLCK);	
#ifdef DEBUG		
		printf("get write lock \n");
#endif		
		// Delay for some time , simulate the RAM writing
		// sleep(1);
		if (HaveRedundEnv) {
			/* switch to next partition for writing */
			otherdev = !curdev;
			if ((fdr = open ((char *)DEVNAME (otherdev), mode)) < 0) {
				fprintf (stderr,
					"Can't open %s: %s\n",
					DEVNAME (otherdev),
					strerror (errno));
				return (-1);
			}
		} else {
			otherdev = curdev;
			fdr = fd;
		}
#ifdef DEBUG		
		printf ("Unlocking flash...\n");
#endif		
#if !defined(RAM_FILE)		
		erase.length = DEVESIZE (otherdev);
		erase.start = DEVOFFSET (otherdev);
//#if !defined(RAM_FILE)		
		ioctl (fdr, MEMUNLOCK, &erase);
#endif
		if (HaveRedundEnv) {
#if !defined(RAM_FILE)		
			erase.length = DEVESIZE (curdev);
			erase.start = DEVOFFSET (curdev);
//#if !defined(RAM_FILE)		
			ioctl (fd, MEMUNLOCK, &erase);
#endif			
			environment.flags = active_flag;
		}
#ifdef DEBUG
		printf ("Done\n");
#endif		
		resid = DEVESIZE (otherdev) - CFG_ENV_SIZE;
		if (resid) {
			if ((data = malloc (resid)) == NULL) {
				fprintf (stderr,
					"Cannot malloc %d bytes: %s\n",
					resid,
					strerror (errno));
				return (-1);
			}
			if (lseek (fdr, DEVOFFSET (otherdev) + CFG_ENV_SIZE, SEEK_SET)
				== -1) {
				fprintf (stderr, "seek error on %s: %s\n",
					DEVNAME (otherdev),
					strerror (errno));
				return (-1);
			}
			if ((rc = read (fdr, data, resid)) != resid) {
				fprintf (stderr,
					"read error on %s: %s\n",
					DEVNAME (otherdev),
					strerror (errno));
				return (-1);
			}
		}
#ifdef DEBUG
		printf ("Erasing old environment...\n");
#endif
#if !defined(RAM_FILE)		
		erase.length = DEVESIZE (otherdev);
		erase.start = DEVOFFSET (otherdev);
//#if !defined(RAM_FILE)		
		if (ioctl (fdr, MEMERASE, &erase) != 0) {
			fprintf (stderr, "MTD erase error on %s: %s\n",
				DEVNAME (otherdev),
				strerror (errno));
			return (-1);
		}
#endif
#ifdef DEBUG		
		printf ("Done\n");
		printf ("Writing environment to %s...\n", DEVNAME (otherdev));
#endif
		if (lseek (fdr, DEVOFFSET (otherdev), SEEK_SET) == -1) {
			fprintf (stderr,
				"seek error on %s: %s\n",
				DEVNAME (otherdev), strerror (errno));
			return (-1);
		}
		if (write (fdr, &environment, len) != len) {
			fprintf (stderr,
				"CRC write error on %s: %s\n",
				DEVNAME (otherdev), strerror (errno));
			return (-1);
		}
		if (write (fdr, environment.data, ENV_SIZE) != ENV_SIZE) {
			fprintf (stderr,
				"Write error on %s: %s\n",
				DEVNAME (otherdev), strerror (errno));
			return (-1);
		}
		if (resid) {
			if (write (fdr, data, resid) != resid) {
				fprintf (stderr,
					"write error on %s: %s\n",
					DEVNAME (curdev), strerror (errno));
				return (-1);
			}
			free (data);
		}
		if (HaveRedundEnv) {
			/* change flag on current active env partition */
			if (lseek (fd, DEVOFFSET (curdev) + sizeof (ulong), SEEK_SET)
				== -1) {
				fprintf (stderr, "seek error on %s: %s\n",
					DEVNAME (curdev), strerror (errno));
				return (-1);
			}
			if (write (fd, &obsolete_flag, sizeof (obsolete_flag)) !=
				sizeof (obsolete_flag)) {
				fprintf (stderr,
					"Write error on %s: %s\n",
					DEVNAME (curdev), strerror (errno));
				return (-1);
			}
		}
#ifdef DEBUG
		printf ("Done\n");
		printf ("Locking ...\n");
#endif
#if !defined(RAM_FILE)		
		erase.length = DEVESIZE (otherdev);
		erase.start = DEVOFFSET (otherdev);
//#if !defined(RAM_FILE)		
		ioctl (fdr, MEMLOCK, &erase);
#endif		
		if (HaveRedundEnv) {
#if !defined(RAM_FILE)					
			erase.length = DEVESIZE (curdev);
			erase.start = DEVOFFSET (curdev);
//#if !defined(RAM_FILE)		
			ioctl (fd, MEMLOCK, &erase);
#endif			
			if (close (fdr)) {
				fprintf (stderr,
					"I/O error on %s: %s\n",
					DEVNAME (otherdev),
					strerror (errno));
				return (-1);
			}
		}
#ifdef DEBUG
		printf ("Done\n");
#endif
	} else {
#ifdef DEBUG		
		printf("getting read lock \n");
#endif		
		getlock(fd, F_RDLCK);
#ifdef DEBUG		
		printf("get read lock\n");
#endif		
		if (lseek (fd, DEVOFFSET (curdev), SEEK_SET) == -1) {
			fprintf (stderr,
				"seek error on %s: %s\n",
				DEVNAME (curdev), strerror (errno));
			close(fd);
			return (-1);
		}
		if (read (fd, &environment, len) != len) {
			fprintf (stderr,
				"CRC read error on %s: ",
				DEVNAME (curdev));
			close(fd);
			return (-1);
		}
		if ((rc = read (fd, environment.data, ENV_SIZE)) != ENV_SIZE) {
			fprintf (stderr,
				"Read error on %s: %s\n",
				DEVNAME (curdev), strerror (errno));
			close(fd);
			return (-1);
		}
	}
	// close will free the lock automaticlly
	if (close (fd)) {
		fprintf (stderr,
			"I/O error on %s: %s\n",
			DEVNAME (curdev), strerror (errno));
		return (-1);
	}

	/* everything ok */
	return (0);
}

/*
 * s1 is either a simple 'name', or a 'name=value' pair.
 * s2 is a 'name=value' pair.
 * If the names match, return the value of s2, else NULL.
 */

static uchar *envmatch (uchar * s1, uchar * s2)
{

	while (*s1 == *s2++)
		if (*s1++ == '=')
			return (s2);
	if (*s1 == '\0' && *(s2 - 1) == '=')
		return (s2);
	return (NULL);
}

/*
 * Prevent confusion if running from erased flash memory
 */
static int env_init (void)
{
	int crc1, crc1_ok;
	uchar *addr1;

	int crc2, crc2_ok;
	uchar flag1, flag2, *addr2;

	if (parse_config ())		/* should fill envdevices */
		return 1;

	if ((addr1 = calloc (1, ENV_SIZE)) == NULL) {
		fprintf (stderr,
			"Not enough memory for environment (%ld bytes)\n",
			ENV_SIZE);
	      	environment.data=0;
		return (errno);
	}

	/* read environment from FLASH to local buffer */
	environment.data = addr1;
	curdev = 0;
	if (flash_io (O_RDONLY)) {
		
	//	return (errno); // Dennis Lee
	}

	crc1_ok = ((crc1 = crc32 (0, environment.data, ENV_SIZE))
			   == environment.crc);
	if (!HaveRedundEnv) {
		if (!crc1_ok) {
			fprintf (stderr,
				"Warning: Bad CRC, using default environment\n");
			environment.data = default_environment;
			free (addr1);
		}
	} else {
		flag1 = environment.flags;

		curdev = 1;
		if ((addr2 = calloc (1, ENV_SIZE)) == NULL) {
			fprintf (stderr,
				"Not enough memory for environment (%ld bytes)\n",
				ENV_SIZE);
			return (errno);
		}
		environment.data = addr2;

		if (flash_io (O_RDONLY)) {
			return (errno);
		}

		crc2_ok = ((crc2 = crc32 (0, environment.data, ENV_SIZE))
				   == environment.crc);
		flag2 = environment.flags;

		if (crc1_ok && !crc2_ok) {
			environment.data = addr1;
			environment.flags = flag1;
			environment.crc = crc1;
			curdev = 0;
			free (addr2);
		} else if (!crc1_ok && crc2_ok) {
			environment.data = addr2;
			environment.flags = flag2;
			environment.crc = crc2;
			curdev = 1;
			free (addr1);
		} else if (!crc1_ok && !crc2_ok) {
			fprintf (stderr,
				"Warning: Bad CRC, using default environment\n");
			environment.data = default_environment;
			curdev = 0;
			free (addr2);
			free (addr1);
		} else if (flag1 == active_flag && flag2 == obsolete_flag) {
			environment.data = addr1;
			environment.flags = flag1;
			environment.crc = crc1;
			curdev = 0;
			free (addr2);
		} else if (flag1 == obsolete_flag && flag2 == active_flag) {
			environment.data = addr2;
			environment.flags = flag2;
			environment.crc = crc2;
			curdev = 1;
			free (addr1);
		} else if (flag1 == flag2) {
			environment.data = addr1;
			environment.flags = flag1;
			environment.crc = crc1;
			curdev = 0;
			free (addr2);
		} else if (flag1 == 0xFF) {
			environment.data = addr1;
			environment.flags = flag1;
			environment.crc = crc1;
			curdev = 0;
			free (addr2);
		} else if (flag2 == 0xFF) {
			environment.data = addr2;
			environment.flags = flag2;
			environment.crc = crc2;
			curdev = 1;
			free (addr1);
		}
	}
	return (0);
}


static int parse_config ()
{
	struct stat st;

#if defined(CONFIG_FILE)
	/* Fills in DEVNAME(), ENVSIZE(), DEVESIZE(). Or don't. */
	if (get_config (CONFIG_FILE)) {
		fprintf (stderr,
			"Cannot parse config file: %s\n", strerror (errno));
		return 1;
	}
#else
	strcpy ((char *)DEVNAME (0), DEVICE1_NAME);
	DEVOFFSET (0) = DEVICE1_OFFSET;
	ENVSIZE (0) = ENV1_SIZE;
	DEVESIZE (0) = DEVICE1_ESIZE;
#ifdef HAVE_REDUND
	strcpy (DEVNAME (1), DEVICE2_NAME);
	DEVOFFSET (1) = DEVICE2_OFFSET;
	ENVSIZE (1) = ENV2_SIZE;
	DEVESIZE (1) = DEVICE2_ESIZE;
	HaveRedundEnv = 1;
#endif
#endif
	if (stat ((char *)DEVNAME (0), &st)) {
		fprintf (stderr,
			"Cannot access MTD device %s: %s\n",
			DEVNAME (0), strerror (errno));
#if defined(RAM_FILE)
		{
		   char buf[256];
		   sprintf(buf,"touch %s\n",DEVNAME(0));
		   if(system(buf))
			fprintf(stderr, "touch fail %s: %s \n",DEVNAME(0),strerror(errno));	   
		}
#else		
		return 1;
#endif		
	}

	if (HaveRedundEnv && stat ((char *)DEVNAME (1), &st)) {
		fprintf (stderr,
			"Cannot access MTD device %s: %s\n",
			DEVNAME (2), strerror (errno));
		return 1;
	}
	return 0;
}

#if defined(CONFIG_FILE)
static int get_config (char *fname)
{
	FILE *fp;
	int i = 0;
	int rc;
	char dump[128];

	if ((fp = fopen (fname, "r")) == NULL) {
		return 1;
	}

	while ((i < 2) && ((rc = fscanf (fp, "%s %lx %lx %lx",
				  DEVNAME (i),
				  &DEVOFFSET (i),
				  &ENVSIZE (i),
				  &DEVESIZE (i)  )) != EOF)) {

		/* Skip incomplete conversions and comment strings */
		if ((rc < 3) || (*DEVNAME (i) == '#')) {
			fgets (dump, sizeof (dump), fp);	/* Consume till end */
			continue;
		}

		i++;
	}
	fclose (fp);

	HaveRedundEnv = i - 1;
	if (!i) {			/* No valid entries found */
		errno = EINVAL;
		return 1;
	} else
		return 0;
}
#endif
