/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Command line user interface to firmware (=U-Boot) environment.
 *
 * Implements:
 *	fw_printenv [ name ... ]
 *		- prints the values of the environment variables
 *		  "name", or the whole environment if no names are
 *		  specified
 *	fw_setenv name [ value ... ]
 *		- If a name without any values is given, the variable
 *		  with this name is deleted from the environment;
 *		  otherwise, all "value" arguments are concatenated,
 *		  separated by sinlge blank characters, and the
 *		  resulting string is assigned to the environment
 *		  variable "name"
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ram_env.h"

#define	CMD_PRINTENV	"ram_printenv"
#define CMD_SETENV	"ram_setenv"
#define CMD_BUFSETENV	"ram_bufsetenv"

int
main(int argc, char *argv[])
{
	char *p;
	char *cmdname = *argv;

	if ((p = strrchr (cmdname, '/')) != NULL) {
		cmdname = p + 1;
	}

	if (strcmp(cmdname, CMD_PRINTENV) == 0) {
			ram_printenv (argc, argv);
			return (EXIT_SUCCESS);

	} else if (strcmp(cmdname, CMD_SETENV) == 0) {

			if (ram_setenv (argc, argv) != 0)
				return (EXIT_FAILURE);

			return (EXIT_SUCCESS);
	}
	else if (strcmp(cmdname, CMD_BUFSETENV) == 0) {
		char *dat1[4] = {"setenv","dennis1","1",0};
		char *dat2[4] = {"setenv","dennis2","2",0};
		char *dat3[4] = {"setenv","dennis3","3",0};
		char *dat4[4] = {"setenv","dennis4","4",0};
		char *dat5[4] = {"setenv","dennis5","5",0};
		
			if (ram_buff_setenv (3, dat1) != 0)
				return (EXIT_FAILURE);
			if (ram_buff_setenv (3, dat2) != 0)
				return (EXIT_FAILURE);
			if (ram_buff_setenv (3, dat3) != 0)
				return (EXIT_FAILURE);
			if (ram_buff_setenv (3, dat4) != 0)
				return (EXIT_FAILURE);
			if (ram_buff_setenv (3, dat5) != 0)
				return (EXIT_FAILURE);
			if (do_ram_commit()!=0)
				return (EXIT_FAILURE);
			return (EXIT_SUCCESS);
	}
	fprintf (stderr,
		"Identity crisis - may be called as `" CMD_PRINTENV
		"' or as `" CMD_SETENV "' but not as `%s'\n",
		cmdname);
	return (EXIT_FAILURE);
}
