#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include "nvram.h"

int
main(int argc, char **argv)
{
    char *base;

    base = argv[1];

    if (argc == 2) {
       if (strstr(base, "init")) {
          /* init system status */
          init();
       }
       else if (strstr(base, "make_wireless_config")) {
          /* make wireless config */
          make_config();
       }
       else if (strstr(base, "start_wan_interface")) {
          /* check wan connection typw */
          wan_interface_up();
       }
       return 0;
    }
    else {
       return 0;
    }
}

int init()
{
   struct ifreq ifr;
   char *ptr;
   int skfd;
   char wl_if_hw[18] = {0};
   char ether_if_hw[18] = {0};
   char wlanbuf[128] = {0};
   int ret;

    if((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
       perror("socket");
       exit(EXIT_FAILURE);
    }

   /* hardware address */
   strncpy(ifr.ifr_name, "ra0", IF_NAMESIZE);
   if(ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) {
	 perror("SIOCGIFHWADDR");
	 return;
   }

   ptr = (char *)&ifr.ifr_addr.sa_data;
   sprintf(wl_if_hw, "%02X:%02X:%02X:%02X:%02X:%02X",
	 	  (ptr[0] & 0377), (ptr[1] & 0377), (ptr[2] & 0377),
	 	  (ptr[3] & 0377), (ptr[4] & 0377), (ptr[5] & 0377));

   sprintf(ether_if_hw, "%02X:%02X:%02X:%02X:%02X:%02X",
	 	  (ptr[0] & 0377), (ptr[1] & 0377), (ptr[2] & 0377),
	 	  (ptr[3] & 0377), (ptr[4] & 0377), (ptr[5] & 0377) - 0x01);

   (void) close(skfd);

   memset(wlanbuf, 0, 128);
   sprintf(wlanbuf, "/sbin/ifconfig eth0 down");
   ret = system(wlanbuf);
   if (ret)
      fprintf(stderr, "command (%s) fail !!!!!\n", wlanbuf);

   memset(wlanbuf, 0, 128);
   sprintf(wlanbuf, "/sbin/ifconfig eth0 hw ether %s", ether_if_hw);
   ret = system(wlanbuf);
   if (ret)
      fprintf(stderr, "command (%s) fail !!!!!\n", wlanbuf);

   memset(wlanbuf, 0, 128);
   sprintf(wlanbuf, "/sbin/ifconfig eth1 down");
   ret = system(wlanbuf);
   if (ret)
      fprintf(stderr, "command (%s) fail !!!!!\n", wlanbuf);

   memset(wlanbuf, 0, 128);
   sprintf(wlanbuf, "/sbin/ifconfig eth1 hw ether %s", wl_if_hw);
   ret = system(wlanbuf);
   if (ret)
      fprintf(stderr, "command (%s) fail !!!!!\n", wlanbuf);
}

int make_config()
{
    FILE *wlanconf;
    char *wordlist=NULL;
    int  i, ssid_num = 1;
    char tx_rate[12], wmm_enable[8];
	char tmpbuf[72], tmp;

    nvram_set("SystemName","RalinkAP");

}	

int wan_interface_up()
{
   char *wordlist = NULL;
   char wlanbuf[128] = {0};
   char wan_ip[16] = {0};
   char wan_mask[16] = {0};
   int  ret = 0;

   nvram_init();

   wordlist = nvram_get("wanConnectionMode");

   if (strcmp(wordlist, "BRIDGE") == 0) {

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl addbr br0");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl setfd br0 1");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl addif br0 eth0");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/ifconfig eth0 0.0.0.0");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl addif br0 eth1");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/ifconfig eth1 0.0.0.0");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      wordlist = nvram_get("lan_ipaddr");
      if (wordlist == NULL)
         strcpy(wan_ip, "192.168.100.1");
      else if (strcmp(wordlist, "") == 0)
      	 strcpy(wan_ip, "192.168.100.1");
      else
         strcpy(wan_ip, wordlist);

      wordlist = nvram_get("lan_netmask");
      if (wordlist == NULL)
         strcpy(wan_mask, "255.255.255.0");
      else if (strcmp(wordlist, "") == 0)
         strcpy(wan_mask, "255.255.255.0");
      else
         strcpy(wan_mask, wordlist);

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/ifconfig br0 %s netmask %s", wan_ip, wan_mask);
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      wordlist = nvram_get("wan_gateway");
      if (wordlist == NULL) {
         memset(wlanbuf, 0, 128);
         sprintf(wlanbuf, "/sbin/route add default gw 192.168.100.254");
         ret = system(wlanbuf);
         if (ret) {
            printf("%s fail !!!!!\n", wlanbuf);
            return 0;
         }
      }
      else if (strcmp(wordlist, "") == 0) {
         memset(wlanbuf, 0, 128);
         sprintf(wlanbuf, "/sbin/route add default gw 192.168.100.254");
         ret = system(wlanbuf);
         if (ret) {
            printf("%s fail !!!!!\n", wlanbuf);
            return 0;
         }
      }
      else {
         memset(wlanbuf, 0, 128);
         sprintf(wlanbuf, "/sbin/route add default gw %s", wordlist);
         ret = system(wlanbuf);
         if (ret) {
            printf("%s fail !!!!!\n", wlanbuf);
            return 0;
         }
      }
   }
   else if (strcmp(wordlist, "DHCP") == 0) {

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/ifconfig eth0 0.0.0.0");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/udhcpc");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl addbr br0");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl setfd br0 1");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl addif br0 eth1");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/ifconfig br0 %s netmask %s", nvram_get("lan_ipaddr"), nvram_get("lan_netmask"));
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/rc.bridge");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }
   }
   else if (strcmp(wordlist, "STATIC") == 0) {

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/ifconfig eth0 0.0.0.0");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      wordlist = nvram_get("wan_ipaddr");
      if (wordlist == NULL)
         strcpy(wan_ip, "192.168.5.201");
      else if (strcmp(wordlist, "") == 0)
      	 strcpy(wan_ip, "192.168.5.201");
      else
         strcpy(wan_ip, wordlist);

      wordlist = nvram_get("wan_netmask");
      if (wordlist == NULL)
         strcpy(wan_mask, "255.255.255.0");
      else if (strcmp(wordlist, "") == 0)
         strcpy(wan_mask, "255.255.255.0");
      else
         strcpy(wan_mask, wordlist);

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/ifconfig eth0 %s netmask %s", wan_ip, wan_mask);
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      wordlist = nvram_get("wan_gateway");
      if (wordlist == NULL) {
         memset(wlanbuf, 0, 128);
         sprintf(wlanbuf, "/sbin/route add default gw 192.168.5.254");
         ret = system(wlanbuf);
         if (ret) {
            printf("%s fail !!!!!\n", wlanbuf);
            return 0;
         }
      }
      else if (strcmp(wordlist, "") == 0) {
         memset(wlanbuf, 0, 128);
         sprintf(wlanbuf, "/sbin/route add default gw 192.168.5.254");
         ret = system(wlanbuf);
         if (ret) {
            printf("%s fail !!!!!\n", wlanbuf);
            return 0;
         }
      }
      else {
         memset(wlanbuf, 0, 128);
         sprintf(wlanbuf, "/sbin/route add default gw %s", wordlist);
         ret = system(wlanbuf);
         if (ret) {
            printf("%s fail !!!!!\n", wlanbuf);
            return 0;
         }
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl addbr br0");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl setfd br0 1");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/brctl addif br0 eth1");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/sbin/ifconfig br0 %s netmask %s", nvram_get("lan_ipaddr"), nvram_get("lan_netmask"));
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }

      memset(wlanbuf, 0, 128);
      sprintf(wlanbuf, "/etc/rc.bridge");
      ret = system(wlanbuf);
      if (ret) {
         printf("%s fail !!!!!\n", wlanbuf);
         return 0;
      }
   }
   else if (strcmp(wordlist, "PPPOE") == 0) {
   }

   nvram_close();
}
